#ifndef _DS_STATE_DESCRIPTOR_H
#define _DS_STATE_DESCRIPTOR_H

#include "../dsString.h"
#include <list>
#include <vector>

class dsVarDescriptor {
public:
    enum Type {
        kNone = 0xFF,
        kInt = 0, kFloat, kBool, kString, kKey, kStateDesc, kCreatable,
        kDouble, kTime, kByte, kShort, kAgeTimeOfDay,
        kVector3 = 50, kPoint3, kRGB, kRGBA, kQuat, kRGB8, kRGBA8,
    };

    enum Flags {
        kInternal = (1<<0),
        kAlwaysNew = (1<<1),
        kVariableLength = (1<<2),
    };

private:
    class _data {
    public:
        dsString fName, fDisplay;
        dsInt32 fType, fCount;
        dsUInt32 fFlags;
        class dsStateDescriptor* fStateDesc;
        int fRefs;

        inline _data(class dsStateDescriptor* stateDesc);
        inline void ref();
        inline void unref();
    } *fData;

public:
    inline dsVarDescriptor(class dsStateDescriptor* stateDesc);
    inline dsVarDescriptor(const dsVarDescriptor& rhs);
    inline ~dsVarDescriptor();

    inline dsVarDescriptor& operator=(const dsVarDescriptor& rhs);
    inline bool operator==(const dsVarDescriptor& rhs);
    inline bool isValid() const;

    inline void setInternal();
    inline void setAlwaysNew();
    inline void setVariableLength();

    inline dsString name() const;
    inline dsString display() const;
    inline dsInt32 type() const;
    inline dsInt32 count() const;
    inline dsUInt32 flags() const;

    inline void setName(const dsString& name);
    inline void setDisplay(const dsString& display);
    inline void setType(dsInt32 type);
    inline void setCount(dsInt32 count);

    static dsInt32 typeFromName(const dsString& typeStr);
};

class dsStateDescriptor {
private:
    class _data {
    public:
        dsString fName;
        dsInt32 fVersion;
        std::list<dsVarDescriptor> fVariables;
        int fRefs;

        inline _data();
        inline void ref();
        inline void unref();
    } *fData;

public:
    inline dsStateDescriptor();
    inline dsStateDescriptor(const dsStateDescriptor& rhs);
    inline ~dsStateDescriptor();

    inline dsStateDescriptor& operator=(const dsStateDescriptor& rhs);
    inline bool operator==(const dsStateDescriptor& rhs);

    void addVariable(const dsVarDescriptor& var);
    dsVarDescriptor operator[](const dsString& name) const;
    std::vector<dsVarDescriptor> variables() const;

    inline dsString name() const;
    inline dsInt32 version() const;

    inline void setName(const dsString& name);
    inline void setVersion(dsInt32 version);
};

#include "dsStateDescriptor.inl"

#endif
