/* dsVarDescriptor */
dsVarDescriptor::_data::_data(class dsStateDescriptor* stateDesc)
               : fCount(0), fFlags(0), fStateDesc(stateDesc), fRefs(1)
{ }

void dsVarDescriptor::_data::ref()
{
    ++fRefs;
}

void dsVarDescriptor::_data::unref()
{
    if (--fRefs == 0)
        delete this;
}

dsVarDescriptor::dsVarDescriptor(class dsStateDescriptor* stateDesc)
               : fData(new _data(stateDesc))
{ }

dsVarDescriptor::dsVarDescriptor(const dsVarDescriptor& rhs)
               : fData(rhs.fData)
{
    fData->ref();
}

dsVarDescriptor::~dsVarDescriptor()
{
    fData->unref();
}

dsVarDescriptor& dsVarDescriptor::operator=(const dsVarDescriptor& rhs)
{
    rhs.fData->ref();
    fData->unref();
    fData = rhs.fData;
    return *this;
}

bool dsVarDescriptor::operator==(const dsVarDescriptor& rhs)
{
    return (fData->fStateDesc == rhs.fData->fStateDesc)
        && (fData->fName == rhs.fData->fName);
}

bool dsVarDescriptor::isValid() const
{
    return (fData->fStateDesc != NULL);
}

void dsVarDescriptor::setInternal()
{
    fData->fFlags |= kInternal;
}

void dsVarDescriptor::setAlwaysNew()
{
    fData->fFlags |= kAlwaysNew;
}

void dsVarDescriptor::setVariableLength()
{
    fData->fFlags |= kVariableLength;
}

dsString dsVarDescriptor::name() const
{
    return fData->fName;
}

dsString dsVarDescriptor::display() const
{
    return fData->fDisplay;
}

dsInt32 dsVarDescriptor::count() const
{
    return fData->fCount;
}

dsInt32 dsVarDescriptor::type() const
{
    return fData->fType;
}

dsUInt32 dsVarDescriptor::flags() const
{
    return fData->fFlags;
}

void dsVarDescriptor::setName(const dsString& name)
{
    fData->fName = name;
}

void dsVarDescriptor::setDisplay(const dsString& display)
{
    fData->fDisplay = display;
}

void dsVarDescriptor::setType(dsInt32 type)
{
    fData->fType = type;
}

void dsVarDescriptor::setCount(dsInt32 count)
{
    fData->fCount = count;
}


/* dsStateDescriptor */
dsStateDescriptor::_data::_data()
                 : fVersion(-1), fRefs(1)
{ }

void dsStateDescriptor::_data::ref()
{
    ++fRefs;
}

void dsStateDescriptor::_data::unref()
{
    if (--fRefs == 0)
        delete this;
}

dsStateDescriptor::dsStateDescriptor()
                 : fData(new _data())
{ }

dsStateDescriptor::dsStateDescriptor(const dsStateDescriptor& rhs)
                 : fData(rhs.fData)
{
    fData->ref();
}

dsStateDescriptor::~dsStateDescriptor()
{
    fData->unref();
}

dsStateDescriptor& dsStateDescriptor::operator=(const dsStateDescriptor& rhs)
{
    rhs.fData->ref();
    fData->unref();
    fData = rhs.fData;
    return *this;
}

bool dsStateDescriptor::operator==(const dsStateDescriptor& rhs)
{
    return (rhs.fData == fData);
}

dsString dsStateDescriptor::name() const
{
    return fData->fName;
}

dsInt32 dsStateDescriptor::version() const
{
    return fData->fVersion;
}

void dsStateDescriptor::setName(const dsString& name)
{
    fData->fName = name;
}

void dsStateDescriptor::setVersion(dsInt32 version)
{
    fData->fVersion = version;
}
