#include "AuthMessages.h"
#include "../vault/dsVaultNode.h"

/* Client -> Server */
static dsNetMsgField Cli2Auth_PingRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Ping time
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Payload
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Cli2Auth_PingRequest)

static dsNetMsgField Cli2Auth_ClientRegisterRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Build ID
};
MAKE_NETMSG(Cli2Auth_ClientRegisterRequest)

static dsNetMsgField Cli2Auth_ClientSetCCRLevel_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // CCR Level
};
MAKE_NETMSG(Cli2Auth_ClientSetCCRLevel)

static dsNetMsgField Cli2Auth_AcctLoginRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Client Challenge
    { kFieldString,    64, sizeof(dsChar)   },  // Account Name
    { kFieldRawData,   20, sizeof(dsUByte)  },  // Password Hash
    { kFieldString,    64, sizeof(dsChar)   },  // Auth Token
    { kFieldString,     8, sizeof(dsChar)   },  // OS
};
MAKE_NETMSG(Cli2Auth_AcctLoginRequest)

static dsNetMsgField Cli2Auth_AcctSetPlayerRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsInt32)  },  // Player ID
};
MAKE_NETMSG(Cli2Auth_AcctSetPlayerRequest)

static dsNetMsgField Cli2Auth_AcctCreateRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldString,    64, sizeof(dsChar)   },  // Account Name
    { kFieldRawData,   20, sizeof(dsUByte)  },  // Password Hash
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Account Flags
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Billing Type
};
MAKE_NETMSG(Cli2Auth_AcctCreateRequest)

static dsNetMsgField Cli2Auth_AcctChangePasswordRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldString,    64, sizeof(dsChar)   },  // Account Name
    { kFieldRawData,   20, sizeof(dsUByte)  },  // Password Hash
};
MAKE_NETMSG(Cli2Auth_AcctChangePasswordRequest)

static dsNetMsgField Cli2Auth_AcctSetRolesRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldString,    64, sizeof(dsChar)   },  // Account Name
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Account Flags
};
MAKE_NETMSG(Cli2Auth_AcctSetRolesRequest)

static dsNetMsgField Cli2Auth_AcctSetBillingTypeRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldString,    64, sizeof(dsChar)   },  // Account Name
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Billing Type
};
MAKE_NETMSG(Cli2Auth_AcctSetBillingTypeRequest)

static dsNetMsgField Cli2Auth_AcctActivateRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldData,      16, sizeof(dsUByte)  },  // Activation Key (UUID)
};
MAKE_NETMSG(Cli2Auth_AcctActivateRequest)

static dsNetMsgField Cli2Auth_AcctCreateFromKeyRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldString,    64, sizeof(dsChar)   },  // Account Name
    { kFieldRawData,   20, sizeof(dsUByte)  },  // Password Hash
    { kFieldData,      16, sizeof(dsUByte)  },  // Key (UUID)
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Billing Type
};
MAKE_NETMSG(Cli2Auth_AcctCreateFromKeyRequest)

static dsNetMsgField Cli2Auth_PlayerDeleteRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Player ID
};
MAKE_NETMSG(Cli2Auth_PlayerDeleteRequest)

static dsNetMsgField Cli2Auth_PlayerCreateRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldString,    40, sizeof(dsChar)   },  // Player Name
    { kFieldString,   260, sizeof(dsChar)   },  // Avatar Shape
    { kFieldString,   260, sizeof(dsChar)   },  // Friend Invite
};
MAKE_NETMSG(Cli2Auth_PlayerCreateRequest)

static dsNetMsgField Cli2Auth_UpgradeVisitorRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Player ID
};
MAKE_NETMSG(Cli2Auth_UpgradeVisitorRequest)

static dsNetMsgField Cli2Auth_SetPlayerBanStatusRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Player ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Banned
};
MAKE_NETMSG(Cli2Auth_SetPlayerBanStatusRequest)

static dsNetMsgField Cli2Auth_KickPlayer_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Player ID
};
MAKE_NETMSG(Cli2Auth_KickPlayer)

static dsNetMsgField Cli2Auth_ChangePlayerNameRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Player ID
    { kFieldString,    40, sizeof(dsChar)   },  // Player Name
};
MAKE_NETMSG(Cli2Auth_ChangePlayerNameRequest)

static dsNetMsgField Cli2Auth_SendFriendInviteRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldData,      16, sizeof(dsUByte)  },  // Invite UUID
    { kFieldString,    64, sizeof(dsChar)   },  // Email address
    { kFieldString,    40, sizeof(dsChar)   },  // To Name
};
MAKE_NETMSG(Cli2Auth_SendFriendInviteRequest)

static dsNetMsgField Cli2Auth_VaultNodeCreate_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Node Data
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Cli2Auth_VaultNodeCreate)

static dsNetMsgField Cli2Auth_VaultNodeFetch_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
};
MAKE_NETMSG(Cli2Auth_VaultNodeFetch)

static dsNetMsgField Cli2Auth_VaultNodeSave_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
    { kFieldData,      16, sizeof(dsUByte)  },  // Revision ID
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Node Data
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Cli2Auth_VaultNodeSave)

static dsNetMsgField Cli2Auth_VaultNodeAdd_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Parent ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Child ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Owner ID
};
MAKE_NETMSG(Cli2Auth_VaultNodeAdd)

static dsNetMsgField Cli2Auth_VaultNodeRemove_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Parent ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Child ID
};
MAKE_NETMSG(Cli2Auth_VaultNodeRemove)

static dsNetMsgField Cli2Auth_VaultFetchNodeRefs_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
};
MAKE_NETMSG(Cli2Auth_VaultFetchNodeRefs)

static dsNetMsgField Cli2Auth_VaultInitAgeRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldData,      16, sizeof(dsUByte)  },  // Age Instance UUID
    { kFieldData,      16, sizeof(dsUByte)  },  // Parent Age UUID
    { kFieldString,   260, sizeof(dsChar)   },  // Age Filename
    { kFieldString,   260, sizeof(dsChar)   },  // Age Instance Name
    { kFieldString,   260, sizeof(dsChar)   },  // Age User Name
    { kFieldString,  1024, sizeof(dsChar)   },  // Age Description
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Age Sequence Number
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Age Language
};
MAKE_NETMSG(Cli2Auth_VaultInitAgeRequest)

static dsNetMsgField Cli2Auth_VaultNodeFind_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Node Data
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Cli2Auth_VaultNodeFind)

static dsNetMsgField Cli2Auth_VaultSetSeen_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Parent ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Child ID
    { kFieldInteger,    0, sizeof(dsUByte)  },  // Seen
};
MAKE_NETMSG(Cli2Auth_VaultSetSeen)

static dsNetMsgField Cli2Auth_VaultSendNode_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Player ID
};
MAKE_NETMSG(Cli2Auth_VaultSendNode)

static dsNetMsgField Cli2Auth_AgeRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldString,    40, sizeof(dsChar)   },  // Age Name
    { kFieldData,      16, sizeof(dsUByte)  },  // Age UUID
};
MAKE_NETMSG(Cli2Auth_AgeRequest)

static dsNetMsgField Cli2Auth_FileListRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldString,   260, sizeof(dsChar)   },  // Directory
    { kFieldString,   256, sizeof(dsChar)   },  // Extension
};
MAKE_NETMSG(Cli2Auth_FileListRequest)

static dsNetMsgField Cli2Auth_FileDownloadRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldString,   260, sizeof(dsChar)   },  // File Name
};
MAKE_NETMSG(Cli2Auth_FileDownloadRequest)

static dsNetMsgField Cli2Auth_FileDownloadChunkAck_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
};
MAKE_NETMSG(Cli2Auth_FileDownloadChunkAck)

static dsNetMsgField Cli2Auth_PropagateBuffer_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Type
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Buffer Data
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Cli2Auth_PropagateBuffer)

static dsNetMsgField Cli2Auth_GetPublicAgeList_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldString,    64, sizeof(dsChar)   },  // Age Filename
};
MAKE_NETMSG(Cli2Auth_GetPublicAgeList)

static dsNetMsgField Cli2Auth_SetAgePublic_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Age Info ID
    { kFieldInteger,    0, sizeof(dsUByte)  },  // Public
};
MAKE_NETMSG(Cli2Auth_SetAgePublic)

static dsNetMsgField Cli2Auth_LogPythonTraceback_Fields[] = {
    { kFieldString,  1024, sizeof(dsChar)   },  // Traceback
};
MAKE_NETMSG(Cli2Auth_LogPythonTraceback)

static dsNetMsgField Cli2Auth_LogStackDump_Fields[] = {
    { kFieldString,  1024, sizeof(dsChar)   },  // Stackdump
};
MAKE_NETMSG(Cli2Auth_LogStackDump)

static dsNetMsgField Cli2Auth_LogClientDebuggerConnect_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Message ID
};
MAKE_NETMSG(Cli2Auth_LogClientDebuggerConnect)

static dsNetMsgField Cli2Auth_ScoreCreate_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Owner ID
    { kFieldString,    64, sizeof(dsChar)   },  // Game Name
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Game Type
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Score Value
};
MAKE_NETMSG(Cli2Auth_ScoreCreate)

static dsNetMsgField Cli2Auth_ScoreDelete_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32)  }, // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32)  }, // Score ID
};
MAKE_NETMSG(Cli2Auth_ScoreDelete)

static dsNetMsgField Cli2Auth_ScoreGetScores_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Owner ID
    { kFieldString,    64, sizeof(dsChar)   },  // Game Name
};
MAKE_NETMSG(Cli2Auth_ScoreGetScores)

static dsNetMsgField Cli2Auth_ScoreAddPoints_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Score ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Num Points
};
MAKE_NETMSG(Cli2Auth_ScoreAddPoints)

static dsNetMsgField Cli2Auth_ScoreTransferPoints_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Source Score ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Dest Score ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Num Points
};
MAKE_NETMSG(Cli2Auth_ScoreTransferPoints)

static dsNetMsgField Cli2Auth_ScoreSetPoints_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Score ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Num Points
};
MAKE_NETMSG(Cli2Auth_ScoreSetPoints)

static dsNetMsgField Cli2Auth_ScoreGetRanks_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Owner ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Score Group
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Parent Folder ID
    { kFieldString,    64, sizeof(dsChar)   },  // Game Name
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Time Period
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Num Results
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Page Number
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Sort Desc
};
MAKE_NETMSG(Cli2Auth_ScoreGetRanks)


/* Server -> Client */
static dsNetMsgField Auth2Cli_PingReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Ping time
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Payload
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Auth2Cli_PingReply)

static dsNetMsgField Auth2Cli_ServerAddr_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Server Address
    { kFieldData,      16, sizeof(dsUByte)  },  // Token (UUID)
};
MAKE_NETMSG(Auth2Cli_ServerAddr)

static dsNetMsgField Auth2Cli_NotifyNewBuild_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // "foo"
};
MAKE_NETMSG(Auth2Cli_NotifyNewBuild)

static dsNetMsgField Auth2Cli_ClientRegisterReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32)  },  // Server Challenge
};
MAKE_NETMSG(Auth2Cli_ClientRegisterReply)

static dsNetMsgField Auth2Cli_AcctLoginReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldData,      16, sizeof(dsUByte)  },  // Account UUID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Account Flags
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Billing Type
    { kFieldInteger,    4, sizeof(dsUInt32) },  // NTD Encryption Key
};
MAKE_NETMSG(Auth2Cli_AcctLoginReply)

static dsNetMsgField Auth2Cli_AcctPlayerInfo_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Player ID
    { kFieldString,    40, sizeof(dsChar)   },  // Player Name
    { kFieldString,    64, sizeof(dsChar)   },  // Avatar Model
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Explorer
};
MAKE_NETMSG(Auth2Cli_AcctPlayerInfo)

static dsNetMsgField Auth2Cli_AcctSetPlayerReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_AcctSetPlayerReply)

static dsNetMsgField Auth2Cli_AcctCreateReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldData,      16, sizeof(dsUByte)  },  // Account UUID
};
MAKE_NETMSG(Auth2Cli_AcctCreateReply)

static dsNetMsgField Auth2Cli_AcctChangePasswordReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_AcctChangePasswordReply)

static dsNetMsgField Auth2Cli_AcctSetRolesReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_AcctSetRolesReply)

static dsNetMsgField Auth2Cli_AcctSetBillingTypeReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_AcctSetBillingTypeReply)

static dsNetMsgField Auth2Cli_AcctActivateReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_AcctActivateReply)

static dsNetMsgField Auth2Cli_AcctCreateFromKeyReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldData,      16, sizeof(dsUByte)  },  // Account UUID
    { kFieldData,      16, sizeof(dsUByte)  },  // Activation Key
};
MAKE_NETMSG(Auth2Cli_AcctCreateFromKeyReply)

static dsNetMsgField Auth2Cli_PlayerCreateReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Player ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Explorer
    { kFieldString,    40, sizeof(dsChar)   },  // Player Name
    { kFieldString,    64, sizeof(dsChar)   },  // Avatar Model
};
MAKE_NETMSG(Auth2Cli_PlayerCreateReply)

static dsNetMsgField Auth2Cli_PlayerDeleteReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_PlayerDeleteReply)

static dsNetMsgField Auth2Cli_UpgradeVisitorReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_UpgradeVisitorReply)

static dsNetMsgField Auth2Cli_SetPlayerBanStatusReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_SetPlayerBanStatusReply)

static dsNetMsgField Auth2Cli_ChangePlayerNameReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_ChangePlayerNameReply)

static dsNetMsgField Auth2Cli_SendFriendInviteReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_SendFriendInviteReply)

static dsNetMsgField Auth2Cli_VaultNodeCreated_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
};
MAKE_NETMSG(Auth2Cli_VaultNodeCreated)

static dsNetMsgField Auth2Cli_VaultNodeFetched_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Node Data
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Auth2Cli_VaultNodeFetched)

static dsNetMsgField Auth2Cli_VaultNodeChanged_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
    { kFieldData,      16, sizeof(dsUByte)  },  // Revision Uuid
};
MAKE_NETMSG(Auth2Cli_VaultNodeChanged)

static dsNetMsgField Auth2Cli_VaultNodeDeleted_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
};
MAKE_NETMSG(Auth2Cli_VaultNodeDeleted)

static dsNetMsgField Auth2Cli_VaultNodeAdded_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Parent ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Child ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Owner ID
};
MAKE_NETMSG(Auth2Cli_VaultNodeAdded)

static dsNetMsgField Auth2Cli_VaultNodeRemoved_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Parent ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Child ID
};
MAKE_NETMSG(Auth2Cli_VaultNodeRemoved)

static dsNetMsgField Auth2Cli_VaultNodeRefsFetched_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32)         },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32)         },  // Result
    { kFieldVarCount,   0, sizeof(dsVaultNodeRef)   },  // Node Refs
    { kFieldVarPtr,     0, 0                        },
};
MAKE_NETMSG(Auth2Cli_VaultNodeRefsFetched)

static dsNetMsgField Auth2Cli_VaultInitAgeReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Age Vault ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Age Info Vault ID
};
MAKE_NETMSG(Auth2Cli_VaultInitAgeReply)

static dsNetMsgField Auth2Cli_VaultNodeFindReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldVarCount,   0, sizeof(dsUInt32) },  // Node ID List
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Auth2Cli_VaultNodeFindReply)

static dsNetMsgField Auth2Cli_VaultSaveNodeReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_VaultSaveNodeReply)

static dsNetMsgField Auth2Cli_VaultAddNodeReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_VaultAddNodeReply)

static dsNetMsgField Auth2Cli_VaultRemoveNodeReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_VaultRemoveNodeReply)

static dsNetMsgField Auth2Cli_AgeReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Age MCP ID
    { kFieldData,      16, sizeof(dsUByte)  },  // Age Instance Uuid
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Age Vault ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Game Server Address
};
MAKE_NETMSG(Auth2Cli_AgeReply)

static dsNetMsgField Auth2Cli_FileListReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldVarCount,   0, sizeof(dsChar)   },  // List Data
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Auth2Cli_FileListReply)

static dsNetMsgField Auth2Cli_FileDownloadChunk_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Total Size
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Chunk Offset
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Chunk Data
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Auth2Cli_FileDownloadChunk)

static dsNetMsgField Auth2Cli_PropagateBuffer_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Message Type
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Buffer Data
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Auth2Cli_PropagateBuffer)

static dsNetMsgField Auth2Cli_KickedOff_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Reason
};
MAKE_NETMSG(Auth2Cli_KickedOff)

static dsNetMsgField Auth2Cli_PublicAgeList_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldVarCount,   0, 0x9A0            },  // Ages (NetAgeInfo)
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Auth2Cli_PublicAgeList)

static dsNetMsgField Auth2Cli_ScoreCreateReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Score ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Created Time
};
MAKE_NETMSG(Auth2Cli_ScoreCreateReply)

static dsNetMsgField Auth2Cli_ScoreDeleteReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_ScoreDeleteReply)

static dsNetMsgField Auth2Cli_ScoreGetScoresReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Score Count
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Score Data
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Auth2Cli_ScoreGetScoresReply)

static dsNetMsgField Auth2Cli_ScoreAddPointsReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_ScoreAddPointsReply)

static dsNetMsgField Auth2Cli_ScoreTransferPointsReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_ScoreTransferPointsReply)

static dsNetMsgField Auth2Cli_ScoreSetPointsReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Auth2Cli_ScoreSetPointsReply)

static dsNetMsgField Auth2Cli_ScoreGetRanksReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Rank Count
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Rank Data
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Auth2Cli_ScoreGetRanksReply)


const dsNetMsg* GET_Cli2Auth(dsUInt32 msgId)
{
    static const dsNetMsg* s_messages[] = {
        &Cli2Auth_PingRequest,
        &Cli2Auth_ClientRegisterRequest,
        &Cli2Auth_ClientSetCCRLevel,
        &Cli2Auth_AcctLoginRequest,
        NULL /* kCli2Auth_AcctSetEulaVersion */,
        NULL /* kCli2Auth_AcctSetDataRequest */,
        &Cli2Auth_AcctSetPlayerRequest,
        &Cli2Auth_AcctCreateRequest,
        &Cli2Auth_AcctChangePasswordRequest,
        &Cli2Auth_AcctSetRolesRequest,
        &Cli2Auth_AcctSetBillingTypeRequest,
        &Cli2Auth_AcctActivateRequest,
        &Cli2Auth_AcctCreateFromKeyRequest,
        &Cli2Auth_PlayerDeleteRequest,
        NULL /* kCli2Auth_PlayerUndeleteRequest */,
        NULL /* kCli2Auth_PlayerSelectRequest */,
        NULL /* kCli2Auth_PlayerRenameRequest */,
        &Cli2Auth_PlayerCreateRequest,
        NULL /* kCli2Auth_PlayerSetStatus */,
        NULL /* kCli2Auth_PlayerChat */,
        &Cli2Auth_UpgradeVisitorRequest,
        &Cli2Auth_SetPlayerBanStatusRequest,
        &Cli2Auth_KickPlayer,
        &Cli2Auth_ChangePlayerNameRequest,
        &Cli2Auth_SendFriendInviteRequest,
        &Cli2Auth_VaultNodeCreate,
        &Cli2Auth_VaultNodeFetch,
        &Cli2Auth_VaultNodeSave,
        NULL /* kCli2Auth_VaultNodeDelete */,
        &Cli2Auth_VaultNodeAdd,
        &Cli2Auth_VaultNodeRemove,
        &Cli2Auth_VaultFetchNodeRefs,
        &Cli2Auth_VaultInitAgeRequest,
        &Cli2Auth_VaultNodeFind,
        &Cli2Auth_VaultSetSeen,
        &Cli2Auth_VaultSendNode,
        &Cli2Auth_AgeRequest,
        &Cli2Auth_FileListRequest,
        &Cli2Auth_FileDownloadRequest,
        &Cli2Auth_FileDownloadChunkAck,
        &Cli2Auth_PropagateBuffer,
        &Cli2Auth_GetPublicAgeList,
        &Cli2Auth_SetAgePublic,
        &Cli2Auth_LogPythonTraceback,
        &Cli2Auth_LogStackDump,
        &Cli2Auth_LogClientDebuggerConnect,
        &Cli2Auth_ScoreCreate,
        &Cli2Auth_ScoreDelete,
        &Cli2Auth_ScoreGetScores,
        &Cli2Auth_ScoreAddPoints,
        &Cli2Auth_ScoreTransferPoints,
        &Cli2Auth_ScoreSetPoints,
        &Cli2Auth_ScoreGetRanks,
    };
    return (msgId < COUNTOF(s_messages, dsNetMsg*) ? s_messages[msgId] : NULL);
}

const dsNetMsg* GET_Auth2Cli(dsUInt32 msgId)
{
    static const dsNetMsg* s_messages[] = {
        &Auth2Cli_PingReply,
        &Auth2Cli_ServerAddr,
        &Auth2Cli_NotifyNewBuild,
        &Auth2Cli_ClientRegisterReply,
        &Auth2Cli_AcctLoginReply,
        NULL /* kAuth2Cli_AcctData */,
        &Auth2Cli_AcctPlayerInfo,
        &Auth2Cli_AcctSetPlayerReply,
        &Auth2Cli_AcctCreateReply,
        &Auth2Cli_AcctChangePasswordReply,
        &Auth2Cli_AcctSetRolesReply,
        &Auth2Cli_AcctSetBillingTypeReply,
        &Auth2Cli_AcctActivateReply,
        &Auth2Cli_AcctCreateFromKeyReply,
        NULL /* kAuth2Cli_PlayerList */,
        NULL /* kAuth2Cli_PlayerChat */,
        &Auth2Cli_PlayerCreateReply,
        &Auth2Cli_PlayerDeleteReply,
        &Auth2Cli_UpgradeVisitorReply,
        &Auth2Cli_SetPlayerBanStatusReply,
        &Auth2Cli_ChangePlayerNameReply,
        &Auth2Cli_SendFriendInviteReply,
        NULL /* kAuth2Cli_Unknown_22 */,
        &Auth2Cli_VaultNodeCreated,
        &Auth2Cli_VaultNodeFetched,
        &Auth2Cli_VaultNodeChanged,
        &Auth2Cli_VaultNodeDeleted,
        &Auth2Cli_VaultNodeAdded,
        &Auth2Cli_VaultNodeRemoved,
        &Auth2Cli_VaultNodeRefsFetched,
        &Auth2Cli_VaultInitAgeReply,
        &Auth2Cli_VaultNodeFindReply,
        &Auth2Cli_VaultSaveNodeReply,
        &Auth2Cli_VaultAddNodeReply,
        &Auth2Cli_VaultRemoveNodeReply,
        &Auth2Cli_AgeReply,
        &Auth2Cli_FileListReply,
        &Auth2Cli_FileDownloadChunk,
        &Auth2Cli_PropagateBuffer,
        &Auth2Cli_KickedOff,
        &Auth2Cli_PublicAgeList,
        &Auth2Cli_ScoreCreateReply,
        &Auth2Cli_ScoreDeleteReply,
        &Auth2Cli_ScoreGetScoresReply,
        &Auth2Cli_ScoreAddPointsReply,
        &Auth2Cli_ScoreTransferPointsReply,
        &Auth2Cli_ScoreSetPointsReply,
        &Auth2Cli_ScoreGetRanksReply,
    };
    return (msgId < COUNTOF(s_messages, dsNetMsg*) ? s_messages[msgId] : NULL);
}
