#include "../CommonSand.h"
#include "../dsUuid.h"
#include "../vault/dsVaultNode.h"

/* Client -> Server */
struct PACKED Cli2Auth_PingRequest {
    dsUInt32 fPingTime, fTransId;
    dsUInt32 fDataSize;
};

struct PACKED Cli2Auth_ClientRegisterRequest {
    dsUInt32 fBuildId;
};

// Cli2Auth_ClientSetCCRLevel [2]

struct Cli2Auth_AcctLoginRequest {
    dsUInt32 fTransId;
    dsUInt32 fClientChallenge;
    dsString fAcctName, fAuthToken, fOS;
    dsSha1Hash fPassHash;
};

// Cli2Auth_AcctSetEulaVersion [4]
// Cli2Auth_AcctSetDataRequest [5]

struct PACKED Cli2Auth_AcctSetPlayerRequest {
    dsUInt32 fTransId;
    dsInt32 fPlayerIdx;
};

// Cli2Auth_AcctCreateRequest [7]
// Cli2Auth_AcctChangePasswordRequest [8]
// Cli2Auth_AcctSetRolesRequest [9]
// Cli2Auth_AcctSetBillingTypeRequest [10]
// Cli2Auth_AcctActivateRequest [11]
// Cli2Auth_AcctCreateFromKeyRequest [12]
// Cli2Auth_PlayerDeleteRequest [13]
// Cli2Auth_PlayerUndeleteRequest [14]
// Cli2Auth_PlayerSelectRequest [15]
// Cli2Auth_PlayerRenameRequest [16]

struct Cli2Auth_PlayerCreateRequest {
    dsUInt32 fTransId;
    dsString fPlayerName, fAvatarShape, fFriendInvite;
};

// Cli2Auth_PlayerSetStatus [18]
// Cli2Auth_PlayerChat [19]
// Cli2Auth_UpgradeVisitorRequest [20]
// Cli2Auth_SetPlayerBanStatusRequest [21]
// Cli2Auth_KickPlayer [22]
// Cli2Auth_ChangePlayerNameRequest [23]
// Cli2Auth_SendFriendInviteRequest [24]

struct Cli2Auth_VaultNodeCreate {
    dsUInt32 fTransId;
    dsUInt32 fNodeSize;
    dsUByte* fNodeData;
};

struct PACKED Cli2Auth_VaultNodeFetch {
    dsUInt32 fTransId;
    dsInt32 fNodeId;
};

struct Cli2Auth_VaultNodeSave {
    dsUInt32 fTransId;
    dsInt32 fNodeId;
    dsUuid fRevisionId;
    dsUInt32 fNodeSize;
    dsUByte* fNodeData;
};

// Cli2Auth_VaultNodeDelete [28]

struct PACKED Cli2Auth_VaultNodeAdd {
    dsUInt32 fTransId;
    dsInt32 fParentIdx, fChildIdx, fOwnerIdx;
};

// Cli2Auth_VaultNodeRemove [30]

struct PACKED Cli2Auth_VaultFetchNodeRefs {
    dsUInt32 fTransId;
    dsInt32 fNodeId;
};

struct Cli2Auth_VaultInitAgeRequest {
    dsUInt32 fTransId;
    dsUuid fAgeUuid, fParentAgeUuid;
    dsString fAgeFilename, fAgeInstName, fAgeUserName, fAgeDesc;
    dsUInt32 fAgeSequenceNumber;
    dsInt32 fAgeLanguage;
};

struct Cli2Auth_VaultNodeFind {
    dsUInt32 fTransId;
    dsUInt32 fNodeSize;
    dsUByte* fNodeData;
};

// Cli2Auth_VaultSetSeen [34]
// Cli2Auth_VaultSendNode [35]

struct Cli2Auth_AgeRequest {
    dsUInt32 fTransId;
    dsString fAgeName;
    dsUuid fAgeUuid;
};

struct Cli2Auth_FileListRequest {
    dsUInt32 fTransId;
    dsString fDirectory, fExt;
};

struct Cli2Auth_FileDownloadRequest {
    dsUInt32 fTransId;
    dsString fFilename;
};

struct PACKED Cli2Auth_FileDownloadChunkAck {
    dsUInt32 fTransId;
};

// Cli2Auth_PropagateBuffer [40]
// Cli2Auth_GetPublicAgeList [41]
// Cli2Auth_SetAgePublic [42]
// Cli2Auth_LogPythonTraceback <Inline>
// Cli2Auth_LogStackDump <Inline>
// Cli2Auth_LogClientDebuggerConnect <Inline>
// Cli2Auth_ScoreCreate [46]
// Cli2Auth_ScoreDelete [47]
// Cli2Auth_ScoreGetScores [48]
// Cli2Auth_ScoreAddPoints [49]
// Cli2Auth_ScoreTransferPoints [50]
// Cli2Auth_ScoreSetPoints [51]
// Cli2Auth_ScoreGetRanks [52]


/* Server -> Client */
#define Auth2Cli_Header \
    dsUInt16 fMessage;

struct PACKED Auth2Cli_PingReply {
    Auth2Cli_Header
    dsUInt32 fPingTime, fTransId;
    dsUInt32 fDataSize;
};

struct PACKED Auth2Cli_ServerAddr {
    Auth2Cli_Header
    dsUInt32 fServerAddr;
    dsUuid fGameUuid;
};

// Auth2Cli_NotifyNewBuild [2]

struct PACKED Auth2Cli_ClientRegisterReply {
    Auth2Cli_Header
    dsUInt32 fChallenge;
};

struct PACKED Auth2Cli_AcctLoginReply {
    Auth2Cli_Header
    dsUInt32 fTransId, fResult;
    dsUuid fAcctUuid;
    dsUInt32 fAcctFlags, fBillingType;
    dsUInt32 fDroidKey[4];
};

// Auth2Cli_AcctData [5]
// Auth2Cli_AcctPlayerInfo <Inline>

struct PACKED Auth2Cli_AcctSetPlayerReply {
    Auth2Cli_Header
    dsUInt32 fTransId, fResult;
};

// Auth2Cli_AcctCreateReply [8]
// Auth2Cli_AcctChangePasswordReply [9]
// Auth2Cli_AcctSetRolesReply [10]
// Auth2Cli_AcctSetBillingTypeReply [11]
// Auth2Cli_AcctActivateReply [12]
// Auth2Cli_AcctCreateFromKeyReply [13]
// Auth2Cli_PlayerList [14]
// Auth2Cli_PlayerChat [15]

struct Auth2Cli_PlayerCreateReply {
    Auth2Cli_Header
    dsUInt32 fTransId, fResult;
    dsInt32 fPlayerIdx, fExplorer;
    dsString fPlayerName, fAvatarShape;
};

// Auth2Cli_PlayerDeleteReply [17]
// Auth2Cli_UpgradeVisitorReply [18]
// Auth2Cli_SetPlayerBanStatusReply [19]
// Auth2Cli_ChangePlayerNameReply [20]
// Auth2Cli_SendFriendInviteReply [21]
// Auth2Cli_Unknown_22 [22]

struct PACKED Auth2Cli_VaultNodeCreated {
    Auth2Cli_Header
    dsUInt32 fTransId, fResult;
    dsInt32 fNodeId;
};

struct Auth2Cli_VaultNodeFetched {
    Auth2Cli_Header
    dsUInt32 fTransId, fResult;
    dsUInt32 fNodeSize;
    dsUByte* fNodeData;
};

struct PACKED Auth2Cli_VaultNodeChanged {
    Auth2Cli_Header
    dsInt32 fNodeId;
    dsUuid fRevisionId;
};

// Auth2Cli_VaultNodeDeleted [26]

struct PACKED Auth2Cli_VaultNodeAdded {
    Auth2Cli_Header
    dsInt32 fParentIdx, fChildIdx, fOwnerIdx;
};

// Auth2Cli_VaultNodeRemoved [28]

struct Auth2Cli_VaultNodeRefsFetched {
    Auth2Cli_Header
    dsUInt32 fTransId, fResult;
    dsUInt32 fNumRefs;
    dsVaultNodeRef* fRefs;
};

struct PACKED Auth2Cli_VaultInitAgeReply {
    Auth2Cli_Header
    dsUInt32 fTransId, fResult;
    dsInt32 fAgeId, fAgeInfoId;
};

struct Auth2Cli_VaultNodeFindReply {
    Auth2Cli_Header
    dsUInt32 fTransId, fResult;
    dsUInt32 fNodeCount;
    dsInt32* fNodeIds;
};

struct PACKED Auth2Cli_VaultSaveNodeReply {
    Auth2Cli_Header
    dsUInt32 fTransId, fResult;
};

struct PACKED Auth2Cli_VaultAddNodeReply {
    Auth2Cli_Header
    dsUInt32 fTransId, fResult;
};

// Auth2Cli_VaultRemoveNodeReply [34]

struct PACKED Auth2Cli_AgeReply {
    Auth2Cli_Header
    dsUInt32 fTransId, fResult;
    dsUInt32 fAgeMcpId;
    dsUuid fAgeInstId;
    dsUInt32 fAgeVaultId, fGameServNode;
};

struct PACKED Auth2Cli_FileListReply {
    Auth2Cli_Header
    dsUInt32 fTransId, fResult;
    dsUInt32 fDataSize;
};

struct PACKED Auth2Cli_FileDownloadChunk {
    Auth2Cli_Header
    dsUInt32 fTransId, fResult;
    dsUInt32 fFileSize, fPosition, fDataSize;
};

// Auth2Cli_PropagateBuffer [38]
// Auth2Cli_KickedOff [39]
// Auth2Cli_PublicAgeList [40]
// Auth2Cli_ScoreCreateReply [41]
// Auth2Cli_ScoreDeleteReply [42]
// Auth2Cli_ScoreGetScoresReply [43]
// Auth2Cli_ScoreAddPointsReply [44]
// Auth2Cli_ScoreTransferPointsReply [45]
// Auth2Cli_ScoreSetPointsReply [46]
// Auth2Cli_ScoreGetRanksReply [47]
