#include "dsAuthData.h"
#include "../dsConfigMgr.h"
#include "../dsLog.h"

/* dsAuthFileInfo */
ssize_t dsAuthFileInfo::writeData(dsChar* buf, size_t maxlen) const
{
    ssize_t maxsave = maxlen;
    size_t len = fFilename.length();
    if (len + 1 > maxlen) {
        dsLog(kLogError, "dsAuthFileInfo: Not enough room in buffer");
        return -1;
    }
    memcpy(buf, fFilename.data(), len * sizeof(dsChar));
    buf[len] = 0;
    buf += (len + 1);
    maxlen -= (len + 1);

    if (3 > maxlen) {
        dsLog(kLogError, "dsAuthFileInfo: Not enough room in buffer");
        return -1;
    }
    buf[0] = (dsChar)(fFileSize >> 16);
    buf[1] = (dsChar)(fFileSize & 0xFFFF);
    buf[2] = 0;
    buf += 3;
    maxlen -= 3;
    return (maxsave - maxlen);
}


/* dsAuthFileList */
void dsAuthFileList::read(const dsString& directory, const dsString& ext)
{
    size_t lineNum = 0;

    dsString path = dsConfigMgr::GetPath("AuthDataDir", directory + "_" + ext + ".list");
    FILE* src = fopen(path.cstr(), "r");
    if (src == NULL) {
        dsLogF(kLogWarning, "dsAuthFileList: Could not get list %s", path.cstr().data());
        fStatus = kNetErrInternalError;
        return;
    }
    char buf[4096];
    while (fgets(buf, 4096, src) != NULL) {
        lineNum++;
        dsString ln(buf);
        ln = ln.trim();
        if (ln.isEmpty() || ln.left(1) == "#")
            continue;
        std::vector<dsString> mfsLine = ln.split(',');
        if (mfsLine.size() != 2) {
            dsLogF(kLogWarning, "dsAuthFileList: Error parsing line %u of %s",
                                lineNum, path.cstr().data());
        }
        dsAuthFileInfo file;
        file.fFilename = mfsLine[0];
        file.fFileSize = mfsLine[1].toUInt();
        fFiles.push_back(file);
    }
    fclose(src);
    fStatus = kNetSuccess;
}

void dsAuthFileList::write(dsChar* buf, size_t len)
{
    for (size_t i=0; i<getNumFiles(); i++) {
        ssize_t w = getFile(i).writeData(buf, len);
        if (w == -1) {
            fStatus = kNetErrInternalError;
            return;
        }
        buf += w;
        len -= w;
    }
    if (len != 1) {
        dsLogF(kLogError, "dsAuthFileList: Incorrect buffer size (%u left)", len);
        fStatus = kNetErrInternalError;
        return;
    }
    *buf = 0;
}

size_t dsAuthFileList::calcTotalSize() const
{
    size_t res = 1;     // Final Z-term
    for (size_t i=0; i<getNumFiles(); i++)
        res += getFile(i).calcSize();
    return res;
}


/* dsAuthFileData */
void dsAuthFileData::read(const dsString& filename)
{
    dsString path = dsConfigMgr::GetPath("AuthDataDir", filename);
    FILE* src = fopen(path.cstr(), "rb");
    if (src == NULL) {
        dsLogF(kLogWarning, "Could not open file %s", filename.cstr().data());
        fStatus = kNetErrInternalError;
        return;
    }
    fseek(src, 0, SEEK_END);
    fFileSize = ftell(src);
    fseek(src, 0, SEEK_SET);
    if (fFileData != NULL)
        delete[] fFileData;
    if (fFileSize > 0) {
        fFileData = new dsUByte[fFileSize];
        fread(fFileData, sizeof(dsUByte), fFileSize, src);
    } else {
        fFileData = NULL;
    }
    fclose(src);
    fStatus = kNetSuccess;
}

void dsAuthFileData::write(dsUByte* buf, size_t len)
{
    if (len + fFilePos > fFileSize) {
        dsLog(kLogWarning, "Requested more data than available");
        len = (fFileSize - fFilePos);
    }
    memcpy(buf, fFileData + fFilePos, len);
    fFilePos += len;
    fStatus = kNetSuccess;
}
