#ifndef _DS_AUTH_SERVER_H
#define _DS_AUTH_SERVER_H

#include "../crypt/dsRC4.h"
#include "../crypt/dsBigInteger.h"
#include "../crypt/dsSha1.h"
#include "../db/dsPostgres.h"
#include "../vault/dsVaultClient.h"
#include "../dsUuid.h"
#include <vector>

struct PACKED AuthConnectHeader {
    dsUInt32 fHeaderSize;
    dsUuid fToken;
};

enum EPrivilegeLevel {
    kPrivBanned = -1,
    kPrivNormal = 0,
    kPrivPrivate = 1,
    kPrivCCR = 2,
    kPrivAdmin = 3,
};

struct dsLoginInfo {
    ENetError fResult;
    EPrivilegeLevel fPrivLevel;
    dsUuid fAcctUuid;
    dsUInt32 fAcctFlags, fBillingType;
};

struct dsPlayerInfo {
    ENetError fResult;
    dsInt32 fPlayerIdx;
    dsString fPlayerName;
    dsString fAvatarShape;
    dsInt32 fExplorer;
};

struct dsAgeInfo {
    ENetError fResult;
    dsInt32 fAgeIdx, fAgeInfoIdx;
    dsUuid fAgeInstId;
};

class dsAuthServer : public dsVaultMsgHandler {
private:
    dsRC4Socket fSock;
    dsUByte fServerSeed[7], fSharedKey[7];
    dsPostgres fAuthDb;
    dsLoginInfo fLoginInfo;
    dsPlayerInfo fPlayerInfo;
    dsVaultClient fVault;

    std::map<dsUInt32, dsUInt32> fVaultTransMap;
    std::map<dsUInt32, dsInt32> fCreateQueue;
    std::map<dsUInt32, std::vector<dsInt32> > fFindQueue;
    std::map<dsUInt32, dsUInt32> fStatusQueue;
    dsMutex fVaultTransMutex;

public:
    inline dsAuthServer(dsSocket* sock);
    inline ~dsAuthServer();

    static pid_t Create(dsSocket* sock);

private:
    void run();
    bool calculateKeys(const dsBigInteger& Y);

    inline dsUInt32 waitForNode(dsUInt32 transId, dsInt32& nodeId);
    inline dsUInt32 waitForNodeList(dsUInt32 transId, std::vector<dsInt32>& nodeList);
    inline dsUInt32 waitForStatus(dsUInt32 transId);

    void login(const char* username, const dsSha1Hash& passhash);
    dsPlayerInfo createPlayer(const char* playerName, const char* avatarShape,
                              const char* friendInvite);
    dsAgeInfo initAge(const dsUuid& ageInstanceUuid, const dsUuid& parentAgeUuid,
                      const char* ageFilename, const char* ageInstName,
                      const char* ageUserName, const char* ageDesc,
                      dsUInt32 ageSeqNumber, dsInt32 ageLanguage);
    dsAgeInfo findAge(const char* ageFilename, const dsUuid& ageUuid);

public:
    virtual void onNodeCreated(const msgparm_t* msg);
    virtual void onNodeFetched(const msgparm_t* msg);
    virtual void onNodeChanged(const msgparm_t* msg);
    //virtual void onNodeDeleted(const msgparm_t* msg);
    virtual void onNodeAdded(const msgparm_t* msg);
    //virtual void onNodeRemoved(const msgparm_t* msg);
    virtual void onNodeRefsFetched(const msgparm_t* msg);
    virtual void onNodeFindReply(const msgparm_t* msg);
    virtual void onSaveNodeReply(const msgparm_t* msg);
    virtual void onAddNodeReply(const msgparm_t* msg);
    //virtual void onRemoveNodeReply(const msgparm_t* msg);
};

#include "dsAuthServer.inl"

#endif
