dsPlayerInfo dsAuthServer::createPlayer(const char* playerName, const char* avatarShape,
                                        const char* friendInvite)
{
    dsPlayerInfo result;
    if (!fAuthDb.isConnected()) {
        result.fResult = kNetErrInternalError;
        return result;
    }

    char safeName[40], safeShape[260], safeInvite[260];
    fAuthDb.escapeString(safeName, playerName, 40);
    fAuthDb.escapeString(safeShape, avatarShape, 260);
    fAuthDb.escapeString(safeInvite, friendInvite, 260);

    // Look for existing name
    dsPostgresResult res = fAuthDb.query(dsString::Format(
        "SELECT * FROM auth.\"Players\" WHERE LOWER(\"PlayerName\")=LOWER('%s')",
        safeName));
    if (!res.isValid()) {
        dsLogF(kLogError, "SQL Error in createPlayer: %s", fAuthDb.getErrorMsg());
        result.fResult = kNetErrInternalError;
        return result;
    } else if (res.getNumRows() != 0) {
        dsLogF(kLogInfo, "Player %s already exists", safeName);
        result.fResult = kNetErrPlayerAlreadyExists;
        return result;
    }

    // Look for space in the user's account
    res = fAuthDb.query(dsString::Format(
        "SELECT * FROM auth.\"Players\" WHERE \"AcctUuid\"='%s'",
        fLoginInfo.fAcctUuid.asString().cstr().data()));
    if (!res.isValid()) {
        dsLogF(kLogError, "SQL Error in createPlayer: %s", fAuthDb.getErrorMsg());
        result.fResult = kNetErrInternalError;
        return result;
    } else {
        int nExplorers = 0, nNonExplorers = 0;
        for (dsUInt64 i=0; i<res.getNumRows(); i++) {
            dsPostgresRow row = res.fetchRow();
            if (row["\"Explorer\""].getInt() != 0)
                nExplorers++;
            else
                nNonExplorers++;
        }
        if (((fLoginInfo.fBillingType & 0x1) != 0 && nExplorers >= 5) ||
            ((fLoginInfo.fBillingType & 0x1) == 0 && nNonExplorers >= 1)) {
            dsLogF(kLogInfo, "Too many players on account %s",
                             fLoginInfo.fAcctUuid.asString().cstr().data());
            result.fResult = kNetErrMaxPlayersOnAcct;
            return result;
        }
    }

    // Create the player on the vault
    dsLogF(kLogInfo, "Creating Player '%s' (%s) I[%s]", safeName, safeShape, safeInvite);
    dsUInt32 transId = getInternalTransId();
    dsVaultNode node;
    node.setTimeNow();
    node.setNodeType(kNodePlayer);
    node.setIString64(0, safeName);
    node.setCreatorIdx(-1);
    node.setInt32(1, (fLoginInfo.fBillingType & 0x1) != 0 ? 1 : 0);
    node.setUuid(0, fLoginInfo.fAcctUuid);
    node.setString64(0, safeShape);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, result.fPlayerIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating Player node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // The PlayerInfo Node
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setNodeType(kNodePlayerInfo);
    node.setCreatorIdx(result.fPlayerIdx);
    node.setUInt32(0, result.fPlayerIdx);
    node.setIString64(0, safeName);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    dsInt32 childIdx;
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating PlayerInfo node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fPlayerIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding PlayerInfo node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    /*
    // Add the PlayerInfo node to the AllPlayers folder
    transId = getInternalTransId();
    node.clearAll();
    node.setNodeType(kNodePlayerInfoList);
    node.setInt32(0, kAllPlayersFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendFindRequest(node)] = transId;
    fVaultTransMutex.unlock();
    std::vector<dsInt32> allPlayersFolder;
    if (waitForNodeList(transId, allPlayersFolder) != kNetSuccess
        || allPlayersFolder.size() == 0) {
        dsLog(kLogError, "Error adding PlayerInfo node to AllPlayers folder");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(allPlayersFolder[0], childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding PlayerInfo node to AllPlayers folder");
        result.fResult = kNetErrInternalError;
        return result;
    }
    */

    // Add the System Node link
    transId = getInternalTransId();
    node.clearAll();
    node.setNodeType(kNodeSystem);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendFindRequest(node)] = transId;
    fVaultTransMutex.unlock();
    std::vector<dsInt32> systemNode;
    if (waitForNodeList(transId, systemNode) != kNetSuccess
        || systemNode.size() == 0) {
        dsLog(kLogError, "Error adding System folder to Player node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fPlayerIdx, systemNode[0], 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding System folder to Player node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Ages I Own Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fPlayerIdx);
    node.setNodeType(kNodeAgeInfoList);
    node.setInt32(0, kAgesIOwnFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating AgesIOwnFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fPlayerIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding AgesIOwnFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Ages I Can Visit Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fPlayerIdx);
    node.setNodeType(kNodeAgeInfoList);
    node.setInt32(0, kAgesICanVisitFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating AgesICanVisitFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fPlayerIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding AgesICanVisitFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Age Journals Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fPlayerIdx);
    node.setNodeType(kNodeFolder);
    node.setInt32(0, kAgeJournalsFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating AgeJournalsFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fPlayerIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding AgeJournalsFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Buddy List
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fPlayerIdx);
    node.setNodeType(kNodePlayerInfoList);
    node.setInt32(0, kBuddyListFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating BuddyListFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fPlayerIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding BuddyListFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // People I Know About Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fPlayerIdx);
    node.setNodeType(kNodePlayerInfoList);
    node.setInt32(0, kPeopleIKnowAboutFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating PeopleIKnowAboutFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fPlayerIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding PeopleIKnowAboutFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Ignore List
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fPlayerIdx);
    node.setNodeType(kNodePlayerInfoList);
    node.setInt32(0, kIgnoreListFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating IgnoreList node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fPlayerIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding IgnoreList node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Inbox Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fPlayerIdx);
    node.setNodeType(kNodeFolder);
    node.setInt32(0, kInboxFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating Inbox node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fPlayerIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding Inbox node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Invite Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fPlayerIdx);
    node.setNodeType(kNodeFolder);
    node.setInt32(0, kPlayerInviteFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating InviteFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fPlayerIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding InviteFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Avatar Outfit Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fPlayerIdx);
    node.setNodeType(kNodeFolder);
    node.setInt32(0, kAvatarOutfitFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating Avatar Outfit Folder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fPlayerIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding Avatar Outfit Folder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Avatar Closet Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fPlayerIdx);
    node.setNodeType(kNodeFolder);
    node.setInt32(0, kAvatarClosetFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating Avatar Closet Folder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fPlayerIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding AvatarCloset Folder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Chronicle Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fPlayerIdx);
    node.setNodeType(kNodeFolder);
    node.setInt32(0, kChronicleFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating Chronicle Folder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fPlayerIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding Chronicle Folder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Create the Auth player entry
    result.fResult = kNetSuccess;
    result.fPlayerName = safeName;
    result.fAvatarShape = safeShape;
    result.fExplorer = (fLoginInfo.fBillingType & 0x1) != 0 ? 1 : 0;
    res = fAuthDb.query("INSERT INTO auth.\"Players\" "
                        "(\"AcctUuid\", \"PlayerIdx\", \"PlayerName\","
                        " \"AvatarShape\", \"Explorer\") VALUES "
                      + dsString::Format("('%s', %d, '%s', '%s', %d) RETURNING idx",
                                         fLoginInfo.fAcctUuid.asString().cstr().data(),
                                         result.fPlayerIdx, safeName, safeShape,
                                         result.fExplorer));
    if (!res.isValid() || res.getNumRows() < 1) {
        dsLogF(kLogError, "Error creating player: %s",
               fAuthDb.getErrorMsg());
        result.fResult = kNetErrInternalError;
        result.fPlayerIdx = 0;
    }
    return result;
}

dsAgeInfo dsAuthServer::initAge(const dsUuid& ageUuid, const dsUuid& parentAgeUuid,
                                const char* ageFilename, const char* ageInstName,
                                const char* ageUserName, const char* ageDesc,
                                dsUInt32 ageSeqNumber, dsInt32 ageLanguage)
{
    dsAgeInfo result;

    char safeFilename[260], safeInstName[260], safeUserName[260], safeDesc[1024];
    fAuthDb.escapeString(safeFilename, ageFilename, 260);
    fAuthDb.escapeString(safeInstName, ageInstName, 260);
    fAuthDb.escapeString(safeUserName, ageUserName, 260);
    fAuthDb.escapeString(safeDesc, ageDesc, 1024);

    // Create the Age node
    dsUInt32 transId = getInternalTransId();
    dsVaultNode node;
    node.setTimeNow();
    node.setNodeType(kNodeAge);
    node.setCreatorUuid(ageUuid);
    node.setCreatorIdx(-1);
    node.setUuid(0, ageUuid);
    node.setUuid(1, parentAgeUuid);
    node.setString64(0, safeFilename);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, result.fAgeIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating Age node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Age Info node
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setNodeType(kNodeAgeInfo);
    node.setCreatorUuid(ageUuid);
    node.setCreatorIdx(result.fAgeIdx);
    node.setInt32(0, ageSeqNumber);
    node.setInt32(1, 0);
    node.setInt32(2, ageLanguage);
    node.setUInt32(0, result.fAgeIdx);
    node.setUInt32(1, 0);
    node.setUInt32(2, 0);
    node.setUuid(0, ageUuid);
    node.setUuid(1, parentAgeUuid);
    node.setString64(1, safeFilename);
    node.setString64(2, safeInstName);
    node.setString64(3, safeUserName);
    node.setText(0, safeDesc);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, result.fAgeInfoIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating AgeInfo node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fAgeIdx, result.fAgeInfoIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding AgeInfo node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Add the System Node link
    transId = getInternalTransId();
    node.clearAll();
    node.setNodeType(kNodeSystem);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendFindRequest(node)] = transId;
    fVaultTransMutex.unlock();
    std::vector<dsInt32> systemNode;
    if (waitForNodeList(transId, systemNode) != kNetSuccess
        || systemNode.size() == 0) {
        dsLog(kLogError, "Error adding System folder to Age node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fAgeIdx, systemNode[0], 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding System folder to Age node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Age Devices Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fAgeIdx);
    node.setNodeType(kNodeFolder);
    node.setInt32(0, kAgeDevicesFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    dsInt32 childIdx;
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating AgeDevicesFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fAgeIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding AgeDevicesFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // PeopleIKnowAbout Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fAgeIdx);
    node.setNodeType(kNodePlayerInfoList);
    node.setInt32(0, kPeopleIKnowAboutFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating PeopleIKnowAboutFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fAgeIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding PeopleIKnowAboutFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // Chronicle Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fAgeIdx);
    node.setNodeType(kNodeFolder);
    node.setInt32(0, kChronicleFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating Chronicle Folder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fAgeIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding Chronicle Folder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // SubAges Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fAgeIdx);
    node.setNodeType(kNodeAgeInfoList);
    node.setInt32(0, kSubAgesFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating SubAgesFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fAgeIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding SubAgesFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // AgeOwners Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fAgeIdx);
    node.setNodeType(kNodePlayerInfoList);
    node.setInt32(0, kAgeOwnersFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating AgeOwnersFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fAgeInfoIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding AgeOwnersFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // CanVisit Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fAgeIdx);
    node.setNodeType(kNodePlayerInfoList);
    node.setInt32(0, kCanVisitFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating CanVisitFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fAgeInfoIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding CanVisitFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // ChildAges Folder
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fAgeIdx);
    node.setNodeType(kNodeAgeInfoList);
    node.setInt32(0, kChildAgesFolder);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating ChildAgesFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    transId = getInternalTransId();
    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendAddRequest(result.fAgeInfoIdx, childIdx, 0)] = transId;
    fVaultTransMutex.unlock();
    if (waitForStatus(transId) != kNetSuccess) {
        dsLog(kLogError, "Error adding ChildAgesFolder node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    // The Local SDL Node
    //TODO

    // The Global SDL Node
    //TODO

    // Game Server Node
    transId = getInternalTransId();
    node.clearAll();
    node.setTimeNow();
    node.setCreatorIdx(result.fAgeIdx);
    node.setNodeType(kNodeGameServer);
    node.setUuid(0, ageUuid);
    node.setString64(0, safeFilename);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendCreateRequest(node)] = transId;
    fVaultTransMutex.unlock();
    if (waitForNode(transId, childIdx) != kNetSuccess) {
        dsLog(kLogError, "Error creating Game Server node");
        result.fResult = kNetErrInternalError;
        return result;
    }

    //TODO: Add Local SDL Node
    //TODO: Add Global SDL Node

    // Now register the Game Server instance
    dsUuid adjustedId = ageUuid;
    adjustedId.fData4[3] = (adjustedId.fData4[2] << 4)
                         | (adjustedId.fData4[3] >> 4);

    dsPostgresResult res = fAuthDb.query("INSERT INTO auth.\"GameServers\" "
        "(\"Filename\", \"InstId\", \"ParentId\", \"ServerId\", \"AgeNode\", "
        " \"AgeInfoNode\", \"RunPid\") VALUES "
        + dsString::Format("('%s', '%s', '%s', '%s', %d, %d, 0)",
                           safeFilename, ageUuid.asString().cstr().data(),
                           parentAgeUuid.asString().cstr().data(),
                           adjustedId.asString().cstr().data(),
                           result.fAgeIdx, result.fAgeInfoIdx)
        + " RETURNING idx");
    if (!res.isValid() || res.getNumRows() < 1) {
        dsLogF(kLogError, "Error creating age: %s",
               fAuthDb.getErrorMsg());
        result.fResult = kNetErrInternalError;
        result.fAgeIdx = 0;
        result.fAgeInfoIdx = 0;
        return result;
    }

    result.fResult = kNetSuccess;
    return result;
}

dsAgeInfo dsAuthServer::findAge(const char* ageFilename, const dsUuid& ageUuid)
{
    char safeFilename[64];
    fAuthDb.escapeString(safeFilename, ageFilename, 64);

    // Find the Age node
    dsAgeInfo result;
    dsPostgresResult res = fAuthDb.query("SELECT * FROM auth.\"GameServers\" "
        + dsString::Format("WHERE \"Filename\"='%s' AND \"ServerId\"='%s'",
                           safeFilename, ageUuid.asString().cstr().data()));
    if (!res.isValid()) {
        dsLogF(kLogError, "SQL Error in findAge: %s", fAuthDb.getErrorMsg());
        result.fResult = kNetErrInternalError;
        return result;
    } else if (res.getNumRows() == 0) {
        dsLogF(kLogError, "Error finding Age %s:%s", safeFilename,
                          ageUuid.asString().data());
        result.fResult = kNetErrAgeNotFound;
        return result;
    }

    result.fResult = kNetSuccess;
    dsPostgresRow row = res.fetchRow();
    result.fAgeIdx = row["\"AgeNode\""].getInt();
    result.fAgeInstId = dsUuid::fromString(row["\"InstId\""].getString());
    return result;

    /*
    dsUInt32 transId = getInternalTransId();
    dsVaultNode node;
    node.setNodeType(kNodeAge);
    node.setUuid(0, ageUuid);
    node.setString64(0, safeFilename);

    fVaultTransMutex.lock();
    fVaultTransMap[fVault.sendFindRequest(node)] = transId;
    fVaultTransMutex.unlock();
    std::vector<dsInt32> ageNode;
    if (waitForNodeList(transId, ageNode) != kNetSuccess
        || ageNode.size() == 0) {
        dsLogF(kLogError, "Error finding Age node for %s:%s", safeFilename,
                          ageUuid.asString().data());
        return kNetErrInternalError;
    }
    ageVaultId = ageNode[0];

    return kNetSuccess;
    */
}

void dsAuthServer::onNodeCreated(const msgparm_t* msg)
{
    fVaultTransMutex.lock();
    std::map<dsUInt32, dsUInt32>::iterator f = fVaultTransMap.find(msg[0]);
    if (f == fVaultTransMap.end()) {
        fVaultTransMutex.unlock();
        return;
    }

    dsUInt32 transId = f->second;
    fVaultTransMap.erase(f);

    if (transId & 0x80000000) {
        // Internal transaction
        fStatusQueue[transId] = msg[1];
        if (msg[1] == kNetSuccess)
            fCreateQueue[transId] = msg[2];
        else
            fCreateQueue[transId] = -1;
    } else {
        // External transaction
        const dsNetMsg* reply = GET_Auth2Cli(kAuth2Cli_VaultNodeCreated);
        msgparm_t* replyBuf = AllocMessage(reply);
        replyBuf[0] = transId;
        replyBuf[1] = msg[1];
        replyBuf[2] = msg[2];
        fSock.sendMsg(replyBuf, reply);
        FreeMessage(replyBuf, reply);
    }
    fVaultTransMutex.unlock();
}

void dsAuthServer::onNodeFetched(const msgparm_t* msg)
{
    fVaultTransMutex.lock();
    std::map<dsUInt32, dsUInt32>::iterator f = fVaultTransMap.find(msg[0]);
    if (f == fVaultTransMap.end()) {
        fVaultTransMutex.unlock();
        return;
    }

    dsUInt32 transId = f->second;
    fVaultTransMap.erase(f);

    if (transId & 0x80000000) {
        // Internal transaction
        fStatusQueue[transId] = msg[1];
        //TODO: Add if needed
    } else {
        // External transaction
        const dsNetMsg* reply = GET_Auth2Cli(kAuth2Cli_VaultNodeFetched);
        msgparm_t* replyBuf = AllocMessage(reply);
        replyBuf[0] = transId;
        replyBuf[1] = msg[1];
        replyBuf[2] = msg[2];
        replyBuf[3] = msg[3];
        fSock.sendMsg(replyBuf, reply);

        // Don't delete twice!
        replyBuf[3] = 0;
        FreeMessage(replyBuf, reply);
    }
    fVaultTransMutex.unlock();
}

void dsAuthServer::onNodeChanged(const msgparm_t* msg)
{
    const dsNetMsg* reply = GET_Auth2Cli(kAuth2Cli_VaultNodeChanged);
    msgparm_t* replyBuf = AllocMessage(reply);
    replyBuf[0] = msg[0];
    memcpy((void*)(replyBuf[1]), (void*)(msg[1]), sizeof(dsUuid));
    fSock.sendMsg(replyBuf, reply);
    FreeMessage(replyBuf, reply);
}

void dsAuthServer::onNodeAdded(const msgparm_t* msg)
{
    const dsNetMsg* reply = GET_Auth2Cli(kAuth2Cli_VaultNodeAdded);
    msgparm_t* replyBuf = AllocMessage(reply);
    replyBuf[0] = msg[0];
    replyBuf[1] = msg[1];
    replyBuf[2] = msg[2];
    fSock.sendMsg(replyBuf, reply);
    FreeMessage(replyBuf, reply);
}

void dsAuthServer::onNodeRefsFetched(const msgparm_t* msg)
{
    fVaultTransMutex.lock();
    std::map<dsUInt32, dsUInt32>::iterator f = fVaultTransMap.find(msg[0]);
    if (f == fVaultTransMap.end()) {
        fVaultTransMutex.unlock();
        return;
    }

    dsUInt32 transId = f->second;
    fVaultTransMap.erase(f);

    if (transId & 0x80000000) {
        // Internal transaction
        fStatusQueue[transId] = msg[1];
        //TODO: Add if needed
    } else {
        // External transaction
        const dsNetMsg* reply = GET_Auth2Cli(kAuth2Cli_VaultNodeRefsFetched);
        msgparm_t* replyBuf = AllocMessage(reply);
        replyBuf[0] = transId;
        replyBuf[1] = msg[1];
        replyBuf[2] = msg[2];
        replyBuf[3] = msg[3];
        fSock.sendMsg(replyBuf, reply);

        // Don't delete twice!
        replyBuf[3] = 0;
        FreeMessage(replyBuf, reply);
    }
    fVaultTransMutex.unlock();
}

void dsAuthServer::onNodeFindReply(const msgparm_t* msg)
{
    fVaultTransMutex.lock();
    std::map<dsUInt32, dsUInt32>::iterator f = fVaultTransMap.find(msg[0]);
    if (f == fVaultTransMap.end()) {
        fVaultTransMutex.unlock();
        return;
    }

    dsUInt32 transId = f->second;
    fVaultTransMap.erase(f);

    if (transId & 0x80000000) {
        // Internal transaction
        fStatusQueue[transId] = msg[1];
        if (msg[1] == kNetSuccess)
            fFindQueue[transId] = std::vector<dsInt32>((dsUInt32*)(msg[3]),
                                                       (dsUInt32*)(msg[3]) + msg[2]);
        else
            fFindQueue[transId] = std::vector<dsInt32>();
    } else {
        // External transaction
        const dsNetMsg* reply = GET_Auth2Cli(kAuth2Cli_VaultNodeFindReply);
        msgparm_t* replyBuf = AllocMessage(reply);
        replyBuf[0] = transId;
        replyBuf[1] = msg[1];
        replyBuf[2] = msg[2];
        replyBuf[3] = msg[3];
        fSock.sendMsg(replyBuf, reply);

        // Don't delete twice!
        replyBuf[3] = 0;
        FreeMessage(replyBuf, reply);
    }
    fVaultTransMutex.unlock();
}

void dsAuthServer::onSaveNodeReply(const msgparm_t* msg)
{
    fVaultTransMutex.lock();
    std::map<dsUInt32, dsUInt32>::iterator f = fVaultTransMap.find(msg[0]);
    if (f == fVaultTransMap.end()) {
        fVaultTransMutex.unlock();
        return;
    }

    dsUInt32 transId = f->second;
    fVaultTransMap.erase(f);

    if (transId & 0x80000000) {
        // Internal transaction
        fStatusQueue[transId] = msg[1];
    } else {
        // External transaction
        const dsNetMsg* reply = GET_Auth2Cli(kAuth2Cli_VaultSaveNodeReply);
        msgparm_t* replyBuf = AllocMessage(reply);
        replyBuf[0] = transId;
        replyBuf[1] = msg[1];
        fSock.sendMsg(replyBuf, reply);
        FreeMessage(replyBuf, reply);
    }
    fVaultTransMutex.unlock();
}

void dsAuthServer::onAddNodeReply(const msgparm_t* msg)
{
    fVaultTransMutex.lock();
    std::map<dsUInt32, dsUInt32>::iterator f = fVaultTransMap.find(msg[0]);
    if (f == fVaultTransMap.end()) {
        fVaultTransMutex.unlock();
        return;
    }

    dsUInt32 transId = f->second;
    fVaultTransMap.erase(f);

    if (transId & 0x80000000) {
        // Internal transaction
        fStatusQueue[transId] = msg[1];
    } else {
        // External transaction
        const dsNetMsg* reply = GET_Auth2Cli(kAuth2Cli_VaultAddNodeReply);
        msgparm_t* replyBuf = AllocMessage(reply);
        replyBuf[0] = transId;
        replyBuf[1] = msg[1];
        fSock.sendMsg(replyBuf, reply);
        FreeMessage(replyBuf, reply);
    }
    fVaultTransMutex.unlock();
}
