#ifndef _DS_BIG_INTEGER_H
#define _DS_BIG_INTEGER_H

#include "../CommonSand.h"
#include <openssl/bn.h>

class dsBigInteger {
protected:
    BIGNUM* fValue;
    BN_CTX* fContext;

public:
    inline dsBigInteger();
    inline dsBigInteger(const dsBigInteger& init);
    inline dsBigInteger(dsUInt32 init);
    inline dsBigInteger(const dsUByte* data, size_t count);
    inline ~dsBigInteger();

    inline dsBigInteger& operator=(const dsBigInteger& init);
    inline dsBigInteger& operator=(dsUInt32 init);

    inline void set(const dsBigInteger& init);
    inline void set(dsUInt32 init);
    inline void set(const dsUByte* data, size_t count);
    inline void setBit(size_t bit, bool on);

    inline size_t getByteSize() const;
    inline size_t getBitSize() const;
    inline void getData(dsUByte*& data, size_t& bytes) const;
    inline bool getBit(size_t bit) const;

    // Comparison
    inline bool operator==(const dsBigInteger& num) const;
    inline bool operator==(dsUInt32 num) const;
    inline bool operator!=(const dsBigInteger& num) const;
    inline bool operator!=(dsUInt32 num) const;
    inline bool operator<(const dsBigInteger& num) const;
    inline bool operator<(dsUInt32 num) const;
    inline bool operator>(const dsBigInteger& num) const;
    inline bool operator>(dsUInt32 num) const;
    inline bool operator<=(const dsBigInteger& num) const;
    inline bool operator<=(dsUInt32 num) const;
    inline bool operator>=(const dsBigInteger& num) const;
    inline bool operator>=(dsUInt32 num) const;

    // Math
    inline dsBigInteger& operator++();
    inline dsBigInteger& operator--();
    inline dsBigInteger operator+(const dsBigInteger& num) const;
    inline dsBigInteger operator-(const dsBigInteger& num) const;
    inline dsBigInteger operator*(const dsBigInteger& num) const;
    inline dsBigInteger operator/(const dsBigInteger& num) const;
    inline dsBigInteger operator%(const dsBigInteger& num) const;
    inline dsBigInteger operator+(dsUInt32 num) const;
    inline dsBigInteger operator-(dsUInt32 num) const;
    inline dsBigInteger operator*(dsUInt32 num) const;
    inline dsBigInteger operator/(dsUInt32 num) const;
    inline dsBigInteger operator%(dsUInt32 num) const;
    inline dsBigInteger operator<<(size_t amt) const;
    inline dsBigInteger operator>>(size_t amt) const;
    inline dsBigInteger& operator+=(const dsBigInteger& num);
    inline dsBigInteger& operator-=(const dsBigInteger& num);
    inline dsBigInteger& operator*=(const dsBigInteger& num);
    inline dsBigInteger& operator/=(const dsBigInteger& num);
    inline dsBigInteger& operator%=(const dsBigInteger& num);
    inline dsBigInteger& operator+=(dsUInt32 num);
    inline dsBigInteger& operator-=(dsUInt32 num);
    inline dsBigInteger& operator*=(dsUInt32 num);
    inline dsBigInteger& operator/=(dsUInt32 num);
    inline dsBigInteger& operator%=(dsUInt32 num);
    inline dsBigInteger& operator<<=(size_t amt);
    inline dsBigInteger& operator>>=(size_t amt);

    inline dsBigInteger PowMod(const dsBigInteger& b, const dsBigInteger& m) const;
    inline dsBigInteger DivMod(const dsBigInteger& div, dsBigInteger& r) const;

    static inline dsBigInteger Random(dsUInt32 bits);
};

/* OpenSSL uses big-endian for their binary buffers... */
#define BYTESWAP_BUFFER(x, size) \
    { \
        dsUByte temp; \
        for (size_t i = 0; i < (size / 2); i++) { \
            temp = x[i]; \
            x[i] = x[size-1-i]; \
            x[size-1-i] = temp; \
        } \
    }

#include "dsBigInteger.inl"

#endif
