#include <cstring>

dsBigInteger::dsBigInteger()
{
    fValue = BN_new();
    fContext = BN_CTX_new();
}

dsBigInteger::dsBigInteger(const dsBigInteger& init)
{
    fValue = BN_new();
    fContext = BN_CTX_new();
    BN_copy(fValue, init.fValue);
}

dsBigInteger::dsBigInteger(dsUInt32 init)
{
    fValue = BN_new();
    fContext = BN_CTX_new();
    BN_set_word(fValue, init);
}

dsBigInteger::dsBigInteger(const dsUByte* data, size_t count)
{
    fValue = BN_new();
    fContext = BN_CTX_new();
    dsUByte* buf = new dsUByte[count];
    memcpy(buf, data, count);
    BYTESWAP_BUFFER(buf, count)
    BN_bin2bn(buf, count, fValue);
    delete[] buf;
}

dsBigInteger::~dsBigInteger()
{
    BN_CTX_free(fContext);
    BN_free(fValue);
}

dsBigInteger& dsBigInteger::operator=(const dsBigInteger& init)
{
    BN_copy(fValue, init.fValue);
    return *this;
}

dsBigInteger& dsBigInteger::operator=(dsUInt32 init)
{
    BN_set_word(fValue, init);
    return *this;
}

void dsBigInteger::set(const dsBigInteger& init)
{
    BN_copy(fValue, init.fValue);
}

void dsBigInteger::set(dsUInt32 init)
{
    BN_set_word(fValue, init);
}

void dsBigInteger::set(const dsUByte* data, size_t count)
{
    dsUByte* buf = new dsUByte[count];
    memcpy(buf, data, count);
    BYTESWAP_BUFFER(buf, count)
    BN_bin2bn(buf, count, fValue);
    delete[] buf;
}

void dsBigInteger::setBit(size_t bit, bool on)
{
    if (on)
        BN_set_bit(fValue, bit);
    else
        BN_clear_bit(fValue, bit);
}

size_t dsBigInteger::getByteSize() const
{
    return BN_num_bytes(fValue);
}

size_t dsBigInteger::getBitSize() const
{
    return BN_num_bits(fValue);
}

void dsBigInteger::getData(dsUByte*& data, size_t& bytes) const
{
    bytes = BN_num_bytes(fValue);
    if (bytes != 0) {
        data = new dsUByte[bytes];
        BN_bn2bin(fValue, data);
        BYTESWAP_BUFFER(data, bytes)
    } else {
        data = NULL;
    }
}

bool dsBigInteger::getBit(size_t bit) const
{
    return BN_is_bit_set(fValue, bit) != 0;
}

bool dsBigInteger::operator==(const dsBigInteger& num) const
{
    return BN_cmp(fValue, num.fValue) == 0;
}

bool dsBigInteger::operator==(dsUInt32 num) const
{
    return BN_is_word(fValue, num) != 0;
}

bool dsBigInteger::operator!=(const dsBigInteger& num) const
{
    return BN_cmp(fValue, num.fValue) != 0;
}

bool dsBigInteger::operator!=(dsUInt32 num) const
{
    return BN_is_word(fValue, num) == 0;
}

bool dsBigInteger::operator<(const dsBigInteger& num) const
{
    return BN_cmp(fValue, num.fValue) < 0;
}

bool dsBigInteger::operator<(dsUInt32 num) const
{
    return BN_get_word(fValue) < num;
}

bool dsBigInteger::operator>(const dsBigInteger& num) const
{
    return BN_cmp(fValue, num.fValue) > 0;
}

bool dsBigInteger::operator>(dsUInt32 num) const
{
    if (BN_num_bits(fValue) > 32)
        return true;
    return BN_get_word(fValue) > num;
}

bool dsBigInteger::operator<=(const dsBigInteger& num) const
{
    return BN_cmp(fValue, num.fValue) <= 0;
}

bool dsBigInteger::operator<=(dsUInt32 num) const
{
    if (BN_num_bits(fValue) > 32)
        return false;
    return BN_get_word(fValue) <= num;
}

bool dsBigInteger::operator>=(const dsBigInteger& num) const
{
    return BN_cmp(fValue, num.fValue) >= 0;
}

bool dsBigInteger::operator>=(dsUInt32 num) const
{
    return BN_get_word(fValue) >= num;
}

dsBigInteger& dsBigInteger::operator++()
{
    BN_add_word(fValue, 1);
    return *this;
}

dsBigInteger& dsBigInteger::operator--()
{
    BN_sub_word(fValue, 1);
    return *this;
}

dsBigInteger dsBigInteger::operator+(const dsBigInteger& num) const
{
    dsBigInteger r;
    BN_add(r.fValue, fValue, num.fValue);
    return r;
}

dsBigInteger dsBigInteger::operator-(const dsBigInteger& num) const
{
    dsBigInteger r;
    BN_sub(r.fValue, fValue, num.fValue);
    return r;
}

dsBigInteger dsBigInteger::operator*(const dsBigInteger& num) const
{
    dsBigInteger r;
    BN_mul(r.fValue, fValue, num.fValue, fContext);
    return r;
}

dsBigInteger dsBigInteger::operator/(const dsBigInteger& num) const
{
    dsBigInteger r;
    BN_div(r.fValue, NULL, fValue, num.fValue, fContext);
    return r;
}

dsBigInteger dsBigInteger::operator%(const dsBigInteger& num) const
{
    dsBigInteger r;
    BN_div(NULL, r.fValue, fValue, num.fValue, fContext);
    return r;
}

dsBigInteger dsBigInteger::operator+(dsUInt32 num) const
{
    dsBigInteger r(*this);
    BN_add_word(r.fValue, num);
    return r;
}

dsBigInteger dsBigInteger::operator-(dsUInt32 num) const
{
    dsBigInteger r(*this);
    BN_sub_word(r.fValue, num);
    return r;
}

dsBigInteger dsBigInteger::operator*(dsUInt32 num) const
{
    dsBigInteger r(*this);
    BN_mul_word(r.fValue, num);
    return r;
}

dsBigInteger dsBigInteger::operator/(dsUInt32 num) const
{
    dsBigInteger r(*this);
    BN_div_word(r.fValue, num);
    return r;
}

dsBigInteger dsBigInteger::operator%(dsUInt32 num) const
{
    return BN_mod_word(fValue, num);
}

dsBigInteger dsBigInteger::operator<<(size_t amt) const
{
    dsBigInteger r(*this);
    BN_lshift(r.fValue, fValue, amt);
    return r;
}

dsBigInteger dsBigInteger::operator>>(size_t amt) const
{
    dsBigInteger r(*this);
    BN_rshift(r.fValue, fValue, amt);
    return r;
}

dsBigInteger& dsBigInteger::operator+=(const dsBigInteger& num)
{
    BN_add(fValue, fValue, num.fValue);
    return *this;
}

dsBigInteger& dsBigInteger::operator-=(const dsBigInteger& num)
{
    dsBigInteger left(*this);
    BN_sub(fValue, left.fValue, num.fValue);
    return *this;
}

dsBigInteger& dsBigInteger::operator*=(const dsBigInteger& num)
{
    BN_mul(fValue, fValue, num.fValue, fContext);
    return *this;
}

dsBigInteger& dsBigInteger::operator/=(const dsBigInteger& num)
{
    dsBigInteger left(*this);
    BN_div(fValue, NULL, left.fValue, num.fValue, fContext);
    return *this;
}

dsBigInteger& dsBigInteger::operator%=(const dsBigInteger& num)
{
    dsBigInteger left(*this);
    BN_div(NULL, fValue, left.fValue, num.fValue, fContext);
    return *this;
}

dsBigInteger& dsBigInteger::operator+=(dsUInt32 num)
{
    BN_add_word(fValue, num);
    return *this;
}

dsBigInteger& dsBigInteger::operator-=(dsUInt32 num)
{
    BN_sub_word(fValue, num);
    return *this;
}

dsBigInteger& dsBigInteger::operator*=(dsUInt32 num)
{
    BN_mul_word(fValue, num);
    return *this;
}

dsBigInteger& dsBigInteger::operator/=(dsUInt32 num)
{
    BN_div_word(fValue, num);
    return *this;
}

dsBigInteger& dsBigInteger::operator%=(dsUInt32 num)
{
    dsUInt32 value = BN_mod_word(fValue, num);
    BN_set_word(fValue, value);
    return *this;
}

dsBigInteger& dsBigInteger::operator<<=(size_t amt)
{
    BN_lshift(fValue, fValue, amt);
    return *this;
}

dsBigInteger& dsBigInteger::operator>>=(size_t amt)
{
    BN_rshift(fValue, fValue, amt);
    return *this;
}

dsBigInteger dsBigInteger::PowMod(const dsBigInteger& b, const dsBigInteger& mod) const
{
    dsBigInteger r;
    BN_mod_exp(r.fValue, fValue, b.fValue, mod.fValue, fContext);
    return r;
}

dsBigInteger dsBigInteger::DivMod(const dsBigInteger& div, dsBigInteger& rem) const
{
    dsBigInteger r;
    BN_div(r.fValue, rem.fValue, fValue, div.fValue, fContext);
    return r;
}

dsBigInteger dsBigInteger::Random(dsUInt32 bits)
{
    dsBigInteger r;
    BN_rand(r.fValue, bits, 0, 0);
    return r;
}
