#include "dsSha1.h"
#include <openssl/sha.h>

void dsSha1Hash::fromString(const dsString& src)
{
    if (src.length() != 40) {
        fData[0] = 0;
        fData[1] = 0;
        fData[2] = 0;
        fData[3] = 0;
        fData[4] = 0;
        return;
    }

    fData[0] = src.mid(0, 8).toUInt(16);
    fData[1] = src.mid(8, 8).toUInt(16);
    fData[2] = src.mid(16, 8).toUInt(16);
    fData[3] = src.mid(24, 8).toUInt(16);
    fData[4] = src.mid(32, 8).toUInt(16);
}

dsString dsSha1Hash::toString() const
{
    return dsString::Format("%08x%08x%08x%08x%08x",
                            fData[0], fData[1], fData[2], fData[3], fData[4]);
}

dsSha1Hash Sha0(const void* src, size_t len)
{
    dsSha1Hash hash;
    SHA((const unsigned char*)src, len, (unsigned char*)hash.fData);
    return hash;
}

dsSha1Hash Sha1(const void* src, size_t len)
{
    dsSha1Hash hash;
    SHA1((const unsigned char*)src, len, (unsigned char*)hash.fData);
    return hash;
}
