#include "dsBigInteger.h"

int main(int argc, char* argv[])
{
    if (argc != 4) {
        printf("Usage: %s shared.key server.key out.key\n", argv[0]);
        return 1;
    }

    FILE* key = fopen(argv[1], "rb");
    if (key == NULL) {
        fprintf(stderr, "Could not open keyfile: %s\n", argv[1]);
        return 1;
    }
    dsUByte sharedSeed[64];
    if (fread(sharedSeed, sizeof(dsUByte), 64, key) != 64) {
        fclose(key);
        fprintf(stderr, "Invalid keyfile: %s\n", argv[1]);
        return 1;
    }
    fclose(key);

    key = fopen(argv[2], "rb");
    if (key == NULL) {
        fprintf(stderr, "Could not open keyfile: %s\n", argv[2]);
        return 1;
    }
    dsUByte serverSeed[64];
    if (fread(serverSeed, sizeof(dsUByte), 64, key) != 64) {
        fclose(key);
        fprintf(stderr, "Invalid keyfile: %s\n", argv[2]);
        return 1;
    }
    fclose(key);

    dsBigInteger share(sharedSeed, 64);
    dsBigInteger serv(serverSeed, 64);
    dsBigInteger x = dsBigInteger(4).PowMod(serv, share);
    dsUByte* xData;
    size_t xSize;
    x.getData(xData, xSize);

    key = fopen(argv[3], "wb");
    if (key == NULL) {
        fprintf(stderr, "Could not open output keyfile: %s\n", argv[3]);
        delete[] xData;
        return 1;
    }
    for (size_t i=xSize; i<64; i++) {
        dsUByte zero = 0;
        if (fwrite(&zero, sizeof(dsUByte), 1, key) != 1) {
            fprintf(stderr, "Error writing to keyfile\n");
            fclose(key);
            delete[] xData;
            return 1;
        }
    }
    if (fwrite(xData, sizeof(dsUByte), xSize, key) != xSize) {
        fprintf(stderr, "Error writing to keyfile\n");
        fclose(key);
        delete[] xData;
        return 1;
    }
    fclose(key);
    delete[] xData;

    printf("Key file created!\n");
    return 0;
}
