-- DirtSand DB initialization script --

SET client_encoding = 'UTF8';
SET standard_conforming_strings = off;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET escape_string_warning = off;

CREATE DATABASE dirtsand WITH TEMPLATE = template0 ENCODING = 'UTF8';
ALTER DATABASE dirtsand OWNER TO dirtsand;

\connect dirtsand

SET client_encoding = 'UTF8';
SET standard_conforming_strings = off;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET escape_string_warning = off;

CREATE SCHEMA auth;
ALTER SCHEMA auth OWNER TO dirtsand;

CREATE SCHEMA vault;
ALTER SCHEMA vault OWNER TO dirtsand;

SET search_path = auth, pg_catalog;
SET default_tablespace = '';
SET default_with_oids = false;

CREATE TABLE "Accounts" (
    idx integer NOT NULL,
    "Login" character varying(64) NOT NULL,
    "PassHash" character(40) NOT NULL,
    "AcctUuid" uuid NOT NULL,
    "AcctFlags" integer DEFAULT 0 NOT NULL,
    "BillingType" integer DEFAULT 0 NOT NULL,
    "PrivLevel" integer DEFAULT 0 NOT NULL
);
ALTER TABLE auth."Accounts" OWNER TO dirtsand;

CREATE SEQUENCE "Accounts_idx_seq"
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE auth."Accounts_idx_seq" OWNER TO dirtsand;
ALTER SEQUENCE "Accounts_idx_seq" OWNED BY "Accounts".idx;
SELECT pg_catalog.setval('"Accounts_idx_seq"', 1, false);

CREATE TABLE "Players" (
    idx integer NOT NULL,
    "AcctUuid" uuid NOT NULL,
    "PlayerIdx" integer NOT NULL,
    "PlayerName" character varying(40) NOT NULL,
    "AvatarShape" character varying(64) NOT NULL,
    "Explorer" integer DEFAULT 1 NOT NULL
);
ALTER TABLE auth."Players" OWNER TO dirtsand;

CREATE SEQUENCE "Players_idx_seq"
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE auth."Players_idx_seq" OWNER TO dirtsand;
ALTER SEQUENCE "Players_idx_seq" OWNED BY "Players".idx;
SELECT pg_catalog.setval('"Players_idx_seq"', 1, false);

CREATE TABLE "GameServers" (
    idx integer NOT NULL,
    "Filename" character varying(64) NOT NULL,
    "InstId" uuid NOT NULL,
    "ParentId" uuid NOT NULL,
    "ServerId" uuid NOT NULL,
    "AgeNode" integer DEFAULT 0 NOT NULL,
    "AgeInfoNode" integer DEFAULT 0 NOT NULL,
    "RunPid" integer DEFAULT 0 NOT NULL
);
ALTER TABLE auth."GameServers" OWNER TO dirtsand;

CREATE SEQUENCE "GameServers_idx_seq"
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE auth."GameServers_idx_seq" OWNER TO dirtsand;
ALTER SEQUENCE "GameServers_idx_seq" OWNED BY "GameServers".idx;
SELECT pg_catalog.setval('"GameServers_idx_seq"', 1, false);

SET search_path = vault, pg_catalog;

CREATE TABLE "NodeRefs" (
    idx integer NOT NULL,
    "ParentIdx" integer NOT NULL,
    "ChildIdx" integer NOT NULL,
    "SaverIdx" integer NOT NULL,
    "SeenIdx" integer DEFAULT 0
);
ALTER TABLE vault."NodeRefs" OWNER TO dirtsand;
CREATE SEQUENCE "NodeRefs_idx_seq"
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE vault."NodeRefs_idx_seq" OWNER TO dirtsand;
ALTER SEQUENCE "NodeRefs_idx_seq" OWNED BY "NodeRefs".idx;
SELECT pg_catalog.setval('"NodeRefs_idx_seq"', 1, false);

CREATE TABLE "Nodes" (
    idx integer NOT NULL,
    "CreateTime" integer DEFAULT 0 NOT NULL,
    "ModifyTime" integer DEFAULT 0 NOT NULL,
    "CreateAgeName" character varying(64),
    "CreateAgeUuid" uuid,
    "CreatorUuid" uuid,
    "CreatorIdx" integer DEFAULT 0 NOT NULL,
    "NodeType" integer NOT NULL,
    "Int32_1" integer,
    "Int32_2" integer,
    "Int32_3" integer,
    "Int32_4" integer,
    "UInt32_1" integer,
    "UInt32_2" integer,
    "UInt32_3" integer,
    "UInt32_4" integer,
    "Uuid_1" uuid,
    "Uuid_2" uuid,
    "Uuid_3" uuid,
    "Uuid_4" uuid,
    "String64_1" character varying(64),
    "String64_2" character varying(64),
    "String64_3" character varying(64),
    "String64_4" character varying(64),
    "String64_5" character varying(64),
    "String64_6" character varying(64),
    "IString64_1" character varying(64),
    "IString64_2" character varying(64),
    "Text_1" character varying(255),
    "Text_2" character varying(255),
    "Blob_1" bytea,
    "Blob_2" bytea
);
ALTER TABLE vault."Nodes" OWNER TO dirtsand;
CREATE SEQUENCE "Nodes_idx_seq"
    INCREMENT BY 1
    NO MAXVALUE
    MINVALUE 10001
    CACHE 1;
ALTER TABLE vault."Nodes_idx_seq" OWNER TO dirtsand;
ALTER SEQUENCE "Nodes_idx_seq" OWNED BY "Nodes".idx;
SELECT pg_catalog.setval('"Nodes_idx_seq"', 10001, false);

SET search_path = auth, pg_catalog;
ALTER TABLE "Accounts" ALTER COLUMN idx SET DEFAULT nextval('"Accounts_idx_seq"'::regclass);
ALTER TABLE "GameServers" ALTER COLUMN idx SET DEFAULT nextval('"GameServers_idx_seq"'::regclass);
ALTER TABLE "Players" ALTER COLUMN idx SET DEFAULT nextval('"Players_idx_seq"'::regclass);

ALTER TABLE ONLY "Accounts"
    ADD CONSTRAINT "Accounts_Login_key" UNIQUE ("Login");
ALTER TABLE ONLY "Accounts"
    ADD CONSTRAINT "Accounts_pkey" PRIMARY KEY (idx);
ALTER TABLE ONLY "Players"
    ADD CONSTRAINT "Players_pkey" PRIMARY KEY (idx);
ALTER TABLE ONLY "GameServers"
    ADD CONSTRAINT "GameServers_pkey" PRIMARY KEY (idx);

SET search_path = vault, pg_catalog;
ALTER TABLE "NodeRefs" ALTER COLUMN idx SET DEFAULT nextval('"NodeRefs_idx_seq"'::regclass);
ALTER TABLE "Nodes" ALTER COLUMN idx SET DEFAULT nextval('"Nodes_idx_seq"'::regclass);

ALTER TABLE ONLY "NodeRefs"
    ADD CONSTRAINT "NodeRefs_pkey" PRIMARY KEY (idx);
ALTER TABLE ONLY "Nodes"
    ADD CONSTRAINT "Nodes_pkey" PRIMARY KEY (idx);

SET search_path = auth, pg_catalog;
CREATE INDEX "Login_Index" ON "Accounts" USING hash ("Login");

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM postgres;
GRANT ALL ON SCHEMA public TO postgres;
GRANT ALL ON SCHEMA public TO PUBLIC;
