#ifndef _DS_POSTGRES_H
#define _DS_POSTGRES_H

#include <postgresql/libpq-fe.h>
#include <vector>
#include "../dsString.h"
#include "../dsUuid.h"

struct dsBlob {
    struct _ref {
        dsUInt32 fSize;
        dsUByte* fData;
        dsUInt32 fRefs;

        inline _ref(const dsUByte* data, dsUInt32 size);
        inline ~_ref();
        inline void ref();
        inline void unref();
    } *fBlob;

    inline dsBlob();
    inline dsBlob(const dsUByte* data, dsUInt32 size);
    inline dsBlob(const dsBlob& init);
    inline ~dsBlob();

    inline void set(const dsUByte* data, dsUInt32 size);
    inline dsBlob& operator=(const dsBlob& other);
    bool operator==(const dsBlob& other) const;

    inline bool isNull() const;
    inline bool isEmpty() const;
    inline dsUInt32 size() const;
    inline dsUByte* data() const;
};

class dsPostgresValue {
protected:
    const char* fValue;
    dsUInt32 fLength;

public:
    inline dsPostgresValue();
    inline dsPostgresValue(const char* value, unsigned long length);
    inline bool isNull() const;

    inline dsInt32 getInt() const;
    inline dsUInt32 getUInt() const;
    inline dsInt64 getLong() const;
    inline dsUInt64 getULong() const;
    inline dsFloat getFloat() const;
    inline dsDouble getDouble() const;
    inline const char* getString() const;
    inline dsBlob getBlob() const;
    inline bool getBool() const;
};

struct dsSQLResultRef {
    PGresult* fData;
    dsUInt32 fDataRefs;

    void ref();
    void unref();
};

class dsPostgresRow {
protected:
    int fRowNum, fNumFields;
    dsSQLResultRef* fData;

public:
    inline dsPostgresRow();
    dsPostgresRow(int rowNum, dsSQLResultRef* data);
    inline ~dsPostgresRow();

    dsPostgresValue operator[](int idx) const;
    dsPostgresValue operator[](const char* idx) const;
};

class dsPostgresResult {
protected:
    dsSQLResultRef* fResult;
    int fCurrentRow;

public:
    inline dsPostgresResult();
    dsPostgresResult(PGresult* res);
    inline dsPostgresResult(const dsPostgresResult& init);
    inline ~dsPostgresResult();

    dsPostgresResult& operator=(PGresult* res);
    inline dsPostgresResult& operator=(const dsPostgresResult& init);
    dsPostgresRow operator[](int idx);

    inline bool isValid() const;
    inline dsUInt64 getNumRows() const;
    dsPostgresRow fetchRow();
};

class dsPostgres {
protected:
    PGconn* fConnection;

public:
    inline dsPostgres();
    inline ~dsPostgres();

    bool connect(const char* host, const char* port,
                 const char* username, const char* password,
                 const char* sslmode, const char* database);
    void close();
    inline bool isConnected() const;
    inline void verifyConnection();
    inline const char* getErrorMsg() const;
    dsPostgresResult query(const dsString& query);

    // Utils:
    dsUuid MakeUuid();
    inline void escapeString(char* dest, const char* source, size_t length);
};

dsString Base64(const dsUByte* src, size_t len);
void UnBase64(const dsString& src, dsUByte*& dest, size_t& len);

#include "dsPostgres.inl"

#endif
