#include <cstring>

/* dsBlob */
dsBlob::_ref::_ref(const dsUByte* data, dsUInt32 size)
      : fSize(size), fRefs(1)
{
    fData = new dsUByte[fSize];
    memcpy(fData, data, fSize);
}

dsBlob::_ref::~_ref()
{
    delete[] fData;
}

void dsBlob::_ref::ref()
{
    ++fRefs;
}

void dsBlob::_ref::unref()
{
    if (--fRefs == 0)
        delete this;
}

dsBlob::dsBlob()
      : fBlob(NULL)
{ }

dsBlob::dsBlob(const dsUByte* data, dsUInt32 size)
      : fBlob(new _ref(data, size))
{ }

dsBlob::dsBlob(const dsBlob& init)
      : fBlob(init.fBlob)
{
    if (fBlob != NULL)
        fBlob->ref();
}

dsBlob::~dsBlob()
{
    if (fBlob != NULL)
        fBlob->unref();
}

void dsBlob::set(const dsUByte* data, dsUInt32 size)
{
    if (fBlob != NULL)
        fBlob->unref();
    if (data != NULL)
        fBlob = new _ref(data, size);
    else
        fBlob = NULL;
}

dsBlob& dsBlob::operator=(const dsBlob& other)
{
    if (other.fBlob != NULL)
        other.fBlob->ref();
    if (fBlob != NULL)
        fBlob->unref();
    fBlob = other.fBlob;
    return *this;
}

bool dsBlob::isNull() const
{
    return (fBlob == NULL);
}

bool dsBlob::isEmpty() const
{
    return isNull() || (fBlob->fSize == 0);
}

dsUInt32 dsBlob::size() const
{
    return isNull() ? 0 : fBlob->fSize;
}

dsUByte* dsBlob::data() const
{
    return isNull() ? NULL : fBlob->fData;
}


/* dsPostgresValue */
dsPostgresValue::dsPostgresValue()
               : fValue(NULL), fLength(0)
{ }

dsPostgresValue::dsPostgresValue(const char* value, unsigned long length)
               : fValue(value), fLength(length)
{ }

bool dsPostgresValue::isNull() const
{
    return (fValue == NULL);
}

dsInt32 dsPostgresValue::getInt() const
{
    return isNull() ? 0 : (dsInt32)strtol(fValue, NULL, 10);
}

dsUInt32 dsPostgresValue::getUInt() const
{
    return isNull() ? 0 : (dsUInt32)strtol(fValue, NULL, 10);
}

dsInt64 dsPostgresValue::getLong() const
{
    return isNull() ? 0LL : (dsInt64)strtoll(fValue, NULL, 10);
}

dsUInt64 dsPostgresValue::getULong() const
{
    return isNull() ? 0ULL : (dsUInt64)strtoll(fValue, NULL, 10);
}

dsFloat dsPostgresValue::getFloat() const
{
    return isNull() ? 0.0f : strtof(fValue, NULL);
}

dsDouble dsPostgresValue::getDouble() const
{
    return isNull() ? 0.0 : strtod(fValue, NULL);
}

const char* dsPostgresValue::getString() const
{
    return isNull() ? "" : fValue;
}

dsBlob dsPostgresValue::getBlob() const
{
    if (isNull())
        return dsBlob();

    size_t size;
    dsUByte* data;
    UnBase64(fValue, data, size);
    dsBlob result(data, size);
    delete[] data;
    return result;
}

bool dsPostgresValue::getBool() const
{
    return isNull() ? false :
           (fValue[0] == 'Y' || fValue[0] == 'y' || fValue[0] == '1' ||
            fValue[0] == 'T' || fValue[0] == 't');
}


/* dsPostgresRow */
dsPostgresRow::dsPostgresRow()
             : fRowNum(0), fNumFields(0), fData(NULL)
{ }

dsPostgresRow::~dsPostgresRow()
{
    fData->unref();
}


/* dsPostgresResult */
dsPostgresResult::dsPostgresResult()
                : fResult(NULL), fCurrentRow(0)
{ }

dsPostgresResult::dsPostgresResult(const dsPostgresResult& init)
                : fResult(init.fResult), fCurrentRow(init.fCurrentRow)
{
    fResult->ref();
}

dsPostgresResult::~dsPostgresResult()
{
    fResult->unref();
}

dsPostgresResult& dsPostgresResult::operator=(const dsPostgresResult& init)
{
    fResult = init.fResult;
    fCurrentRow = init.fCurrentRow;
    fResult->ref();
    return *this;
}

bool dsPostgresResult::isValid() const
{
    return (fResult != NULL);
}

dsUInt64 dsPostgresResult::getNumRows() const
{
    return isValid() ? (dsUInt64)PQntuples(fResult->fData) : 0;
}


/* dsPostgres */
dsPostgres::dsPostgres()
          : fConnection(NULL)
{ }

dsPostgres::~dsPostgres()
{
    close();
}

bool dsPostgres::isConnected() const
{
    return (fConnection != NULL);
}

void dsPostgres::verifyConnection()
{
    if (PQstatus(fConnection) == CONNECTION_BAD)
        PQreset(fConnection);
}

const char* dsPostgres::getErrorMsg() const
{
    return isConnected() ? PQerrorMessage(fConnection) : "dsPostgres: Not Connected";
}

void dsPostgres::escapeString(char* dest, const char* source, size_t length)
{
    PQescapeStringConn(fConnection, dest, source, length, NULL);
}
