#include "dsConfigMgr.h"

std::map<dsString, dsString> dsConfigMgr::fConfig;
dsString dsConfigMgr::fBaseDir;
dsMutex dsConfigMgr::fConfigMutex;

bool dsConfigMgr::LoadConfig(const char* filename)
{
    fConfigMutex.lock();
    fConfig.empty();

    FILE* cfg = fopen(filename, "r");
    if (cfg == NULL) {
        fprintf(stderr, "Error: Could not open config file %s\n", filename);
        fConfigMutex.unlock();
        return false;
    }

    size_t lineNum = 0;
    char buf[4096];
    while (fgets(buf, 4096, cfg) != NULL) {
        lineNum++;
        dsString ln(buf);
        ln = ln.beforeFirst('#').trim();
        if (ln.isEmpty())
            continue;
        std::vector<dsString> line = ln.split('=', 1);
        if (line.size() == 2)
            fConfig[line[0]] = line[1];
    }
    fclose(cfg);
    fConfigMutex.unlock();

    fBaseDir = GetPath("BaseDir");
    if (!fBaseDir.isEmpty() && fBaseDir[fBaseDir.length()-1] != '/')
        fBaseDir += "/";

    return true;
}

dsString dsConfigMgr::GetString(const dsString& name, const dsString& def)
{
    dsString str = def;

    fConfigMutex.lock();
    std::map<dsString, dsString>::iterator it = fConfig.find(name);
    if (it != fConfig.end())
        str = it->second;
    fConfigMutex.unlock();
    return str;
}

dsString dsConfigMgr::GetPath(const dsString& name, const dsString& filename)
{
    dsString path;
    dsString rel = GetString(name);
    if (!rel.isEmpty() && rel[0] == '/')
        path = rel;
    else
        path = fBaseDir + rel;

    if (!filename.isEmpty()) {
        if (!path.isEmpty() && path[path.length()-1] != '/')
            path += "/";
        path += filename;
    }
    return path;
}

dsUInt32 dsConfigMgr::GetUInt(const dsString& name, dsUInt32 def)
{
    dsUInt32 value = def;

    fConfigMutex.lock();
    std::map<dsString, dsString>::iterator it = fConfig.find(name);
    if (it != fConfig.end())
        value = it->second.toUInt();
    fConfigMutex.unlock();
    return value;
}

dsInt32 dsConfigMgr::GetInt(const dsString& name, dsInt32 def)
{
    dsInt32 value = def;

    fConfigMutex.lock();
    std::map<dsString, dsString>::iterator it = fConfig.find(name);
    if (it != fConfig.end())
        value = it->second.toInt();
    fConfigMutex.unlock();
    return value;
}
