#include "dsLog.h"
#include "dsConfigMgr.h"
#include <ctime>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <cstring>

const char* dsLogProcessor::kLogTypeNames[] = {
    "ERROR", "Warning", "Info", "Debug"
};

dsLogProcessor* dsLogProcessor::fCurrentLogger = NULL;

dsLogProcessor::~dsLogProcessor()
{
    if (fCurrentLogger == this)
        fCurrentLogger = NULL;
    if (fAddrName != NULL)
        free(fAddrName);
    if (fFileDesc != NULL)
        fclose(fFileDesc);
}

dsLogProcessor* dsLogProcessor::Create(const char* basename, ELogType maxLevel)
{
    dsLogProcessor* lp = new dsLogProcessor();
    lp->fAddrName = NULL;
    lp->fMaxLevel = maxLevel;

    time_t now = time(NULL);
    struct tm local;
    tzset();
    localtime_r(&now, &local);
    char timeBuf[32];
    strftime(timeBuf, 32, "%Y%m%d%H%M%S", &local);

    char* logFile;
    if (asprintf(&logFile, "%s.%s-%d.log", basename, timeBuf, getpid()) < 0) {
        fprintf(stderr, "Error creating log file %s!\n", logFile);
        abort();
    }
    lp->fFileDesc = fopen(logFile, "w");
    if (lp->fFileDesc == NULL) {
        fprintf(stderr, "Error creating log file %s!\n", logFile);
        fprintf(stderr, "Error returned was: %s\n", strerror(errno));
        abort();
    }
    free(logFile);
    return lp;
}

void dsLogProcessor::setAddress(const char* addr)
{
    if (fAddrName != NULL)
        free(fAddrName);
    fAddrName = (addr != NULL) ? strdup(addr) : NULL;
}

void dsLogProcessor::write(ELogType type, const char* msg)
{
    if (this == NULL) {
        // In case we forgot to set up a logger, or if the logger goes bad
        fprintf(stderr, "%s: %s\n", kLogTypeNames[type], msg);
        return;
    }

    if ((int)type > (int)fMaxLevel)
        return;

    fLogMutex.lock();
    time_t now = time(NULL);
    struct tm local;
    tzset();
    localtime_r(&now, &local);
    char timeBuf[32];
    strftime(timeBuf, 32, "%Y-%m-%d %H:%M:%S", &local);

    if (fAddrName != NULL)
        fprintf(fFileDesc, "[%s] [%s] %s: %s\n", timeBuf, fAddrName, kLogTypeNames[type], msg);
    else
        fprintf(fFileDesc, "[%s] %s: %s\n", timeBuf, kLogTypeNames[type], msg);
    fflush(fFileDesc);
    fLogMutex.unlock();
}
