#include "dsNetMsg.h"
#include "dsString.h"

static msgparm_t AllocBasic(dsUInt32 size, dsUInt32 count)
{
    // This works for float and double too, because they're the same
    // size as dsUInt32 and dsUInt64...

    if (size == 1)
        return (msgparm_t)(new dsUByte[count]);
    else if (size == 2)
        return (msgparm_t)(new dsUInt16[count]);
    else if (size == 4)
        return (msgparm_t)(new dsUInt32[count]);
    else if (size == 8)
        return (msgparm_t)(new dsUInt64[count]);
    else
        return 0;
}

msgparm_t* AllocMessage(const dsNetMsg* msg)
{
    if (msg == NULL)
        return NULL;

    msgparm_t* data = new msgparm_t[msg->fFieldCount];
    for (size_t i=0; i<msg->fFieldCount; i++) {
        const dsNetMsgField* field = &msg->fFields[i];
        switch (field->fType) {
        case kFieldInteger:
        case kFieldFloat:
            if (field->fCount == 0)
                // Single Value
                data[i] = 0;
            else
                // Array Value
                data[i] = AllocBasic(field->fSize, field->fCount);
            break;
        case kFieldVarPtr:
        case kFieldRawVarPtr:
        case kFieldString:
        case kFieldVarCount:
            // Variable array -- to be allocated later
            data[i] = 0;
            break;
        case kFieldData:
        case kFieldPtr:
        case kFieldRawData:
        case kFieldRawPtr:
            // Fixed-size array
            data[i] = (msgparm_t)(new dsUByte[field->fSize * field->fCount]);
            break;
        }
    }
    return data;
}

void FreeMessage(msgparm_t* data, const dsNetMsg* msg)
{
    if (data == NULL)
        return;

    for (size_t i=0; i<msg->fFieldCount; i++) {
        const dsNetMsgField* field = &msg->fFields[i];
        switch (field->fType) {
        case kFieldInteger:
        case kFieldFloat:
            if (field->fCount != 0)
                // Array Value
                delete[] (dsUByte*)data[i];
            break;
        case kFieldVarCount:
            // Value only, do nothing
            break;
        case kFieldVarPtr:
        case kFieldRawVarPtr:
        case kFieldString:
            // Variable array
            if (data[i] != 0)
                delete[] (dsUByte*)data[i];
            break;
        case kFieldData:
        case kFieldPtr:
        case kFieldRawData:
        case kFieldRawPtr:
            // Fixed-size array
            delete[] (dsUByte*)data[i];
            break;
        }
    }
    delete[] data;
}

size_t MessageSize(const msgparm_t* data, const dsNetMsg* msg)
{
    size_t bufSize = 0;
    dsUInt32 size = 0;
    dsUInt32 count = 0;

    for (dsUInt32 i=0; i<msg->fFieldCount; i++) {
        const dsNetMsgField* field = &msg->fFields[i];
        switch (field->fType) {
        case kFieldInteger:
        case kFieldFloat:
            if (field->fCount == 0)
                bufSize += field->fSize;
            else
                bufSize += field->fSize * field->fCount;
            break;
        case kFieldString:
            bufSize += sizeof(dsUInt16);
            bufSize += dsStrLen((dsChar*)(data[i])) * sizeof(dsChar);
            break;
        case kFieldVarCount:
            bufSize += sizeof(dsUInt32);
            count = data[i];
            size = field->fSize;
            break;
        case kFieldVarPtr:
        case kFieldRawVarPtr:
            bufSize += count * size;
            count = 0;
            size = 0;
            break;
        case kFieldData:
        case kFieldPtr:
        case kFieldRawData:
        case kFieldRawPtr:
            bufSize += field->fCount * field->fSize;
            break;
        }
    }
    return bufSize;
}
