#ifndef _DS_SOCKET_H
#define _DS_SOCKET_H

#include "CommonSand.h"
#include "dsString.h"
#include "dsThread.h"
#include <cstdlib>
#include <list>

class dsSocket {
protected:
    int fSockHandle;

public:
    inline dsSocket();
    inline dsSocket(int handle);
    inline ~dsSocket();

    dsString getRemoteIpStr() const;
    inline int getHandle() const;

    bool connect(const char* address, dsUInt16 port);
    bool bind(dsUInt16 port);
    dsSocket* listen(int backlog);
    void close(bool force=false);
    inline void unlink();
    inline void link(int handle);

    ssize_t send(const void* buffer, size_t size);
    ssize_t recv(void* buffer, size_t size);
    ssize_t peek(void* buffer, size_t size);
    ssize_t rsize();

    static dsUInt32 GetAddress(const char* addrName);
};

class dsAsyncSocket {
private:
    class _async : public dsThread {
    public:
        struct _datum {
            dsUByte* fData;
            size_t fSize;
        };

        std::list<_datum> fSendQueue, fRecvQueue;
        dsMutex fSockMutex;
        dsSocket* fSock;
        size_t fReadPos;

    public:
        inline _async();
        ~_async();
        inline virtual void initialize();
        inline virtual void finalize();

        inline void flush();

    protected:
        virtual void run();
    } fAsyncIO;

public:
    inline dsAsyncSocket(dsSocket* sock);   // Steals the dsSocket
    inline ~dsAsyncSocket();

    ssize_t send(const void* buffer, size_t size);
    ssize_t recv(void* buffer, size_t size);
    ssize_t peek(void* buffer, size_t size);
    inline void flush();
    inline bool readAvailable() const;
    inline bool waitForData();
    inline bool isConnected() const;

    inline void close(bool force=false);
};

#include "dsSocket.inl"

/* This is a global listen socket for the server.  It's global so we can
   close it whenever we spawn a child process */
extern dsSocket s_listenSock;

#endif
