#include "dsString.h"
#include <list>
#include <wctype.h>

dsString& dsString::setFrom(const dsChar* str, size_t len)
{
    if (fString != NULL)
        fString->unref();

    if (str == NULL) {
        fString = NULL;
        return *this;
    }

    if (len == 0xFFFFFFFF) {
        len = 0;
        while (str[len] != 0)
            len++;
    }
    if (len == 0) {
        fString = NULL;
        return *this;
    }

    dsChar* data = new dsChar[len + 1];
    memcpy(data, str, len * sizeof(dsChar));
    data[len] = 0;
    fString = new _str(data, len);
    return *this;
}

dsString& dsString::setFrom(const char* str, size_t len)
{
    if (fString != NULL)
        fString->unref();

    if (str == NULL) {
        fString = NULL;
        return *this;
    }

    if (len == 0xFFFFFFFF) {
        len = 0;
        while (str[len] != 0)
            len++;
    }
    if (len == 0) {
        fString = NULL;
        return *this;
    }

    dsChar* data = new dsChar[len + 1];
    for (size_t i=0; i<len; i++)
        data[i] = (dsChar)str[i];
    data[len] = 0;
    fString = new _str(data, len);
    return *this;
}

dsString& dsString::compact()
{
    if (fString == NULL) {
        return *this;
    } else if (fString->fStringData[0] == 0) {
        fString->unref();
        fString = NULL;
        return *this;
    }

    // Works so long as the string is always constructed safely
    for (size_t i=1; i<fString->fLength; i++) {
        // Check to < fLength, so that strings who are already compacted won't
        //   need to be modified
        if (fString->fStringData[i] == 0) {
            dsChar* newStr = new dsChar[i+1];
            memcpy(newStr, fString->fStringData, i * sizeof(dsChar));
            newStr[i] = 0;
            fString->unref();
            fString = new _str(newStr, i);
            return *this;
        }
    }
    return *this;
}

dsString& dsString::operator+=(const dsString& right)
{
    if (right.isEmpty())
        return *this;
    if (isEmpty())
        return operator=(right);

    size_t newLen = fString->fLength + right.fString->fLength;
    dsChar* newStr = NULL;
    if (newLen != 0) {
        newStr = new dsChar[newLen + 1];
        memcpy(newStr, fString->fStringData, fString->fLength * sizeof(dsChar));
        memcpy(newStr + fString->fLength, right.fString->fStringData, right.fString->fLength * sizeof(dsChar));
        newStr[newLen] = 0;
    }
    if (fString != NULL)
        fString->unref();
    fString = new _str(newStr, newLen);
    return *this;
}

dsString operator+(const dsString& left, const dsString& right)
{
    if (left.isEmpty())
        return right;
    if (right.isEmpty())
        return left;

    dsString str;
    size_t newLength = left.fString->fLength + right.fString->fLength;
    dsChar* data = new dsChar[newLength + 1];
    memcpy(data, left.fString->fStringData, left.fString->fLength * sizeof(dsChar));
    memcpy(data + left.fString->fLength, right.fString->fStringData, right.fString->fLength * sizeof(dsChar));
    data[newLength] = 0;
    str.fString = new dsString::_str(data, newLength);
    return str;
}

int dsString::compare(const dsString& cmp) const
{
    if (fString == cmp.fString)
        return 0;
    if (fString == NULL)
        return -1;
    if (cmp.fString == NULL)
        return 1;
    const dsChar* left = fString->fStringData;
    const dsChar* right = cmp.fString->fStringData;
    while (*left && *right) {
        if (*left != *right)
            return ((int)*left - (int)*right);
        left++, right++;
    }
    return (*left == *right) ? 0 : ((int)*left - (int)*right);
}

static inline int icmpch(dsChar left, dsChar right)
{
    if (left == right)
        return 0;
    if (left >= 'a' && right <= 'z')
        left -= (dsChar)('a' - 'A');
    if (right >= 'a' && right <= 'z')
        right -= (dsChar)('a' - 'A');
    return left - right;
}

int dsString::icompare(const dsString& cmp) const
{
    if (fString == cmp.fString)
        return 0;
    if (fString == NULL)
        return -1;
    if (cmp.fString == NULL)
        return 1;
    const dsChar* left = fString->fStringData;
    const dsChar* right = cmp.fString->fStringData;
    while (*left && *right) {
        int cmp = icmpch(*left, *right);
        if (cmp != 0)
            return cmp;
        left++, right++;
    }
    return icmpch(*left, *right);
}

void dsString::copyTo(dsChar** strPtr, size_t* length) const
{
    if (isEmpty()) {
        *strPtr = new dsChar[1];
        (*strPtr)[0] = 0;
        *length = 0;
        return;
    }

    *length = fString->fLength;
    *strPtr = new dsChar[fString->fLength + 1];
    memcpy(*strPtr, fString->fStringData, fString->fLength * sizeof(dsChar));
    (*strPtr)[fString->fLength] = 0;
}

void dsString::copyTo(char** strPtr, size_t* length) const
{
    if (isEmpty()) {
        *strPtr = new char[1];
        (*strPtr)[0] = 0;
        *length = 0;
        return;
    }

    *length = fString->fLength;
    *strPtr = new char[fString->fLength + 1];
    char* cp = *strPtr;
    for (size_t i = 0; i < fString->fLength; i++, cp++) {
        if (fString->fStringData[i] < 0x100)
            *cp = fString->fStringData[i];
        else
            *cp = '?';
    }
    (*strPtr)[fString->fLength] = 0;
}

std::vector<dsString> dsString::split(dsChar sep, size_t max) const
{
    if (isEmpty())
        return std::vector<dsString>();
    std::list<dsString> result;
    const dsChar* start = fString->fStringData;
    size_t len = 0;
    while (*(start + len) != 0 && max > 0) {
        if (*(start + len) == sep) {
            result.push_back(dsString(start, len));
            start += len + 1;
            len = 0;
            max--;
        } else {
            len++;
        }
    }
    if (*start != 0) {
        // Put the rest of the string in the final position
        len = 0;
        while (*(start + len) != 0)
            len++;
        result.push_back(dsString(start, len));
    }
    return std::vector<dsString>(result.begin(), result.end());
}

dsString dsString::beforeFirst(dsChar sep) const
{
    if (isNull())
        return dsString();
    for (size_t i=0; i<fString->fLength; i++) {
        if (fString->fStringData[i] == sep)
            return left(i);
    }
    return *this;
}

dsString dsString::beforeLast(dsChar sep) const
{
    if (isNull())
        return dsString();
    for (size_t i=fString->fLength; i>0; i--) {
        if (fString->fStringData[i-1] == sep)
            return left(i-1);
    }
    return dsString();
}

dsString dsString::afterFirst(dsChar sep) const
{
    if (isNull())
        return dsString();
    for (size_t i=0; i<fString->fLength; i++) {
        if (fString->fStringData[i] == sep)
            return mid(i+1);
    }
    return dsString();
}

dsString dsString::afterLast(dsChar sep) const
{
    if (isNull())
        return dsString();
    for (size_t i=fString->fLength; i>0; i--) {
        if (fString->fStringData[i-1] == sep)
            return mid(i);
    }
    return *this;
}

dsString dsString::mid(size_t start, size_t len) const
{
    if (isEmpty())
        return dsString();

    if (len > (fString->fLength - start))
        len = fString->fLength - start;
    return dsString(fString->fStringData + start, len);
}

dsString dsString::replace(const dsString& src, const dsString& dest) const
{
    if (isEmpty() || src.isEmpty())
        return *this;

    dsString result;
    size_t start=0, idx=0;
    while (idx <= (fString->fLength - src.fString->fLength)) {
        if (memcmp(fString->fStringData + idx, src.fString->fStringData, src.fString->fLength * sizeof(dsChar)) == 0) {
            result += mid(start, (idx - start)) + dest;
            idx += src.fString->fLength;
            start = idx;
        } else {
            idx++;
        }
    }
    return result + mid(start, (idx - start) + src.fString->fLength);
}

dsString dsString::ltrim() const
{
    if (isEmpty())
        return dsString();

    size_t idx = 0;
    while ((idx < fString->fLength) && (fString->fStringData[idx] == (dsChar)' ' ||
                                        fString->fStringData[idx] == (dsChar)'\t' ||
                                        fString->fStringData[idx] == (dsChar)'\n' ||
                                        fString->fStringData[idx] == (dsChar)'\r'))
        idx++;
    return mid(idx);
}

dsString dsString::rtrim() const
{
    if (isEmpty())
        return dsString();

    size_t len = fString->fLength;
    while ((len > 0) && (fString->fStringData[len-1] == (dsChar)' ' ||
                         fString->fStringData[len-1] == (dsChar)'\t' ||
                         fString->fStringData[len-1] == (dsChar)'\n' ||
                         fString->fStringData[len-1] == (dsChar)'\r'))
        len--;
    return mid(0, len);
}

dsString dsString::trim() const
{
    if (isEmpty())
        return dsString();

    size_t start = 0, end = fString->fLength;
    while ((start < fString->fLength) && (fString->fStringData[start] == (dsChar)' ' ||
                                          fString->fStringData[start] == (dsChar)'\t' ||
                                          fString->fStringData[start] == (dsChar)'\n' ||
                                          fString->fStringData[start] == (dsChar)'\r'))
        start++;
    while ((end > 0) && (fString->fStringData[end-1] == (dsChar)' ' ||
                         fString->fStringData[end-1] == (dsChar)'\t' ||
                         fString->fStringData[end-1] == (dsChar)'\n' ||
                         fString->fStringData[end-1] == (dsChar)'\r'))
        end--;
    return mid(start, end - start);
}

static inline int inbase(dsChar ch, int base)
{
    if (ch < '0' || ch > 'z')
        return -1;
    int result;
    if (ch - '0' < 10) {
        result = (ch - '0');
    } else if (ch - 'A' < 26) {
        result = (ch - 'A') + 10;
    } else if (ch - 'a' < 26) {
        result = (ch - 'a') + 10;
    } else {
        return -1;
    }
    return (result < base) ? result : -1;
}

dsUInt32 dsString::toUInt(int base) const
{
    if (isEmpty())
        return 0;

    const dsChar* cp = fString->fStringData;
    if (base == 0) {
        if (*cp == '0') {
            cp++;
            if (*cp == 'x') {
                cp++;
                base = 16;
            } else {
                base = 8;
            }
        } else {
            base = 10;
        }
    } else if (base < 2 || base > 36) {
        return 0;
    }
    dsUInt32 result = 0;
    int val;
    while ((val = inbase(*cp++, base)) >= 0) {
        result *= base;
        result += val;
    }
    return result;
}

dsInt32 dsString::toInt(int base) const
{
    if (isEmpty())
        return 0;

    dsInt32 sign = 1;
    const dsChar* cp = fString->fStringData;
    if (*cp == '+') {
        cp++;
    } else if (*cp == '-') {
        sign = -1;
        cp++;
    }
    if (base == 0) {
        if (*cp == '0') {
            cp++;
            if (*cp == 'x') {
                cp++;
                base = 16;
            } else {
                base = 8;
            }
        } else {
            base = 10;
        }
    } else if (base < 2 || base > 36) {
        return 0;
    }
    dsInt32 result = 0;
    int val;
    while ((val = inbase(*cp++, base)) >= 0) {
        result *= base;
        result += val;
    }
    return result * sign;
}

void dsString::toLower()
{
    if (isEmpty())
        return;

    dsChar* cp = fString->fStringData;
    while (*cp != 0) {
        *cp = towlower(*cp);
        ++cp;
    }
}

void dsString::toUpper()
{
    if (isEmpty())
        return;

    dsChar* cp = fString->fStringData;
    while (*cp != 0) {
        *cp = towupper(*cp);
        ++cp;
    }
}
