#ifndef _DS_STRING_H
#define _DS_STRING_H

#include "CommonSand.h"
#include <cstdarg>
#include <vector>

class dsString {
public:
    class CharBuffer {
    protected:
        struct _buf {
            char* fBuffer;
            size_t fLength;
            int fRefs;

            inline _buf(char* str, size_t length);
            inline ~_buf();
            inline void ref();
            inline void unref();
        } *fBuffer;

    public:
        inline CharBuffer(char* str, size_t length);
        inline CharBuffer(const CharBuffer& cpy);
        inline ~CharBuffer();

        inline operator const char*() const;
        inline const char* data() const;
        inline CharBuffer& operator=(const CharBuffer& cpy);
    };

protected:
    struct _str {
        dsChar* fStringData;
        size_t fLength;
        int fRefs;

        inline _str(dsChar* data, size_t len);
        inline ~_str();
        inline void ref();
        inline void unref();
    } *fString;

public:
    inline dsString();
    inline dsString(const dsString& cpy);
    inline dsString(const dsChar* str, size_t len=0xFFFFFFFF);
    inline dsString(const char* str, size_t len=0xFFFFFFFF);
    inline ~dsString();

    inline dsString& operator=(const dsString& src);
    inline dsString& operator=(const dsChar* str);
    inline dsString& operator=(const char* str);
    inline dsString& setFrom(const dsString& src);
    dsString& setFrom(const dsChar* str, size_t len=0xFFFFFFFF);
    dsString& setFrom(const char* str, size_t len=0xFFFFFFFF);
    dsString& compact();

    dsString& operator+=(const dsString& right);
    inline dsString& operator+=(const dsChar* right);
    inline dsString& operator+=(const char* right);

    int compare(const dsString& cmp) const;
    int icompare(const dsString& cmp) const;
    inline int compare(const dsChar* cmp) const;
    inline int icompare(const dsChar* cmp) const;
    inline int compare(const char* cmp) const;
    inline int icompare(const char* cmp) const;
    inline bool operator==(const dsString& cmp) const;
    inline bool operator==(const dsChar* cmp) const;
    inline bool operator==(const char* cmp) const;
    inline bool operator!=(const dsString& cmp) const;
    inline bool operator!=(const dsChar* cmp) const;
    inline bool operator!=(const char* cmp) const;
    inline bool operator<(const dsString& cmp) const;   // For std::map

    inline dsChar operator[](size_t idx) const;
    inline dsChar& operator[](size_t idx);
    inline const dsChar* data() const;
    inline dsChar* data();
    inline CharBuffer cstr() const;
    void copyTo(dsChar** strPtr, size_t* length) const;
    void copyTo(char** strPtr, size_t* length) const;

    inline bool isEmpty() const;
    inline bool isNull() const;
    inline size_t length() const;

    std::vector<dsString> split(dsChar sep, size_t max=0xFFFFFFFF) const;
    dsString beforeFirst(dsChar sep) const;
    dsString beforeLast(dsChar sep) const;
    dsString afterFirst(dsChar sep) const;
    dsString afterLast(dsChar sep) const;
    dsString mid(size_t start, size_t len=0xFFFFFFFF) const;
    inline dsString left(size_t len) const;
    inline dsString right(size_t len) const;
    dsString replace(const dsString& src, const dsString& dest) const;

    dsString ltrim() const;
    dsString rtrim() const;
    dsString trim() const;

    dsUInt32 toUInt(int base=0) const;
    dsInt32 toInt(int base=0) const;

    void toLower();
    void toUpper();

    static inline dsString Format(const char* fmt, ...);
    static inline dsString FormatV(const char* fmt, va_list args);

private:
    friend dsString operator+(const dsString& left, const dsString& right);
};

dsString operator+(const dsString& left, const dsString& right);
inline dsString operator+(const dsString& left, const dsChar* right);
inline dsString operator+(const dsString& left, const char* right);
inline dsString operator+(const dsChar* left, const dsString& right);
inline dsString operator+(const char* left, const dsString& right);

inline size_t dsStrLen(const dsChar* str);

#include "dsString.inl"

#endif
