#include <cstring>
#include <cstdio>
#include <cstdlib>

/* CharBuffer */
dsString::CharBuffer::_buf::_buf(char* str, size_t length)
        : fLength(length), fRefs(1)
{
    if (str != NULL) {
        fBuffer = str;
        fBuffer[fLength] = 0;
    } else {
        fBuffer = new char[1];
        fBuffer[0] = 0;
    }
}

dsString::CharBuffer::_buf::~_buf()
{
    delete[] fBuffer;
}

void dsString::CharBuffer::_buf::ref()
{
    ++fRefs;
}

void dsString::CharBuffer::_buf::unref()
{
    if (--fRefs == 0)
        delete this;
}

dsString::CharBuffer::CharBuffer(char* str, size_t length)
{
    fBuffer = new _buf(str, length);
}

dsString::CharBuffer::CharBuffer(const CharBuffer& cpy)
        : fBuffer(cpy.fBuffer)
{
    fBuffer->ref();
}

dsString::CharBuffer::~CharBuffer()
{
    fBuffer->unref();
}

dsString::CharBuffer::operator const char*() const
{
    return fBuffer->fBuffer;
}

const char* dsString::CharBuffer::data() const
{
    return fBuffer->fBuffer;
}

dsString::CharBuffer& dsString::CharBuffer::operator=(const CharBuffer& cpy)
{
    cpy.fBuffer->ref();
    fBuffer->unref();
    fBuffer = cpy.fBuffer;
    return *this;
}


/* dsString */
dsString::_str::_str(dsChar* data, size_t len)
        : fStringData(data), fLength(len), fRefs(1)
{ }

dsString::_str::~_str()
{
    delete[] fStringData;
}

void dsString::_str::ref()
{
    ++fRefs;
}

void dsString::_str::unref()
{
    if (--fRefs == 0)
        delete this;
}

dsString::dsString()
        : fString(NULL)
{ }

dsString::dsString(const dsString& cpy)
        : fString(cpy.fString)
{
    if (fString != NULL)
        fString->ref();
}

dsString::dsString(const dsChar* str, size_t len)
        : fString(NULL)
{
    setFrom(str, len);
}

dsString::dsString(const char* str, size_t len)
        : fString(NULL)
{
    setFrom(str, len);
}

dsString::~dsString()
{
    if (fString != NULL)
        fString->unref();
}

dsString& dsString::operator=(const dsString& src)
{
    return setFrom(src);
}

dsString& dsString::operator=(const dsChar* str)
{
    return setFrom(str);
}

dsString& dsString::operator=(const char* str)
{
    return setFrom(str);
}

dsString& dsString::setFrom(const dsString& src)
{
    if (src.fString != NULL)
        src.fString->ref();
    if (fString != NULL)
        fString->unref();
    fString = src.fString;
    return *this;
}

dsString& dsString::operator+=(const dsChar* right)
{
    return operator+=(dsString(right));
}

dsString& dsString::operator+=(const char* right)
{
    return operator+=(dsString(right));
}

int dsString::compare(const dsChar* cmp) const
{
    return compare(dsString(cmp));
}

int dsString::icompare(const dsChar* cmp) const
{
    return icompare(dsString(cmp));
}

int dsString::compare(const char* cmp) const
{
    return compare(dsString(cmp));
}

int dsString::icompare(const char* cmp) const
{
    return compare(dsString(cmp));
}

bool dsString::operator==(const dsString& cmp) const
{
    return (compare(cmp) == 0);
}

bool dsString::operator==(const dsChar* cmp) const
{
    return (compare(cmp) == 0);
}

bool dsString::operator==(const char* cmp) const
{
    return (compare(cmp) == 0);
}

bool dsString::operator!=(const dsString& cmp) const
{
    return (compare(cmp) != 0);
}

bool dsString::operator!=(const dsChar* cmp) const
{
    return (compare(cmp) != 0);
}

bool dsString::operator!=(const char* cmp) const
{
    return (compare(cmp) != 0);
}

bool dsString::operator<(const dsString& cmp) const
{
    return (compare(cmp) < 0);
}

dsChar dsString::operator[](size_t idx) const
{
    if (!isNull())
        return fString->fStringData[idx];
    return (dsChar)0;
}

dsChar& dsString::operator[](size_t idx)
{
    if (!isNull())
        return fString->fStringData[idx];
    return *(dsChar*)NULL;
}

static dsChar s_emptyStr[] = { 0 };

const dsChar* dsString::data() const
{
    return isNull() ? s_emptyStr : fString->fStringData;
}

dsChar* dsString::data()
{
    return isNull() ? s_emptyStr : fString->fStringData;
}

dsString::CharBuffer dsString::cstr() const
{
    char* buf;
    size_t bufLen;
    copyTo(&buf, &bufLen);
    return CharBuffer(buf, bufLen);
}

bool dsString::isEmpty() const
{
    return isNull() || (fString->fLength == 0);
}

bool dsString::isNull() const
{
    return (fString == NULL);
}

size_t dsString::length() const
{
    return isNull() ? 0 : fString->fLength;
}

dsString dsString::left(size_t len) const
{
    return mid(0, len);
}

dsString dsString::right(size_t len) const
{
    if (fString != NULL)
        return mid(fString->fLength - len);
    return dsString();
}

dsString dsString::Format(const char* fmt, ...)
{
    va_list argptr;
    va_start(argptr, fmt);
    dsString str = FormatV(fmt, argptr);
    va_end(argptr);
    return str;
}

dsString dsString::FormatV(const char* fmt, va_list args)
{
    char* buf;
    int len = vasprintf(&buf, fmt, args);
    if (len == -1)
        return dsString();
    dsString str(buf, len);
    free(buf);
    return str;
}

dsString operator+(const dsString& left, const dsChar* right)
{
    return left + dsString(right);
}

dsString operator+(const dsString& left, const char* right)
{
    return left + dsString(right);
}

dsString operator+(const dsChar* left, const dsString& right)
{
    return dsString(left) + right;
}

dsString operator+(const char* left, const dsString& right)
{
    return dsString(left) + right;
}

size_t dsStrLen(const dsChar* str)
{
    size_t len = 0;
    while (str[len] != 0)
        len++;
    return len;
}
