#include "dsManifest.h"
#include "../dsLog.h"
#include "../dsConfigMgr.h"

/* dsMfsEntry */
ssize_t dsMfsEntry::writeData(dsChar* buf, size_t maxlen) const
{
    size_t maxsave = maxlen;
    size_t len = fFilename.length();
    if (len + 1 > maxlen) {
        dsLog(kLogError, "dsMfsEntry: Not enough room in buffer");
        return -1;
    }
    memcpy(buf, fFilename.data(), len * sizeof(dsChar));
    buf[len] = 0;
    buf += (len + 1);
    maxlen -= (len + 1);

    len = fDownloadName.length();
    if (len + 1 > maxlen) {
        dsLog(kLogError, "dsMfsEntry: Not enough room in buffer");
        return -1;
    }
    memcpy(buf, fDownloadName.data(), len * sizeof(dsChar));
    buf[len] = 0;
    buf += (len + 1);
    maxlen -= (len + 1);

    len = fHash.length();
    if (len != 32) {
        dsLog(kLogError, "dsMfsEntry: Incorrect MD5 string length");
        return -1;
    }
    if (len + 1 > maxlen) {
        dsLog(kLogError, "dsMfsEntry: Not enough room in buffer");
        return -1;
    }
    memcpy(buf, fHash.data(), len * sizeof(dsChar));
    buf[len] = 0;
    buf += (len + 1);
    maxlen -= (len + 1);

    len = fCompressedHash.length();
    if (len != 32) {
        dsLog(kLogError, "dsMfsEntry: Incorrect MD5 string length");
        return -1;
    }
    if (len + 1 > maxlen) {
        dsLog(kLogError, "dsMfsEntry: Not enough room in buffer");
        return -1;
    }
    memcpy(buf, fCompressedHash.data(), len * sizeof(dsChar));
    buf[len] = 0;
    buf += (len + 1);
    maxlen -= (len + 1);

    if (3 > maxlen) {
        dsLog(kLogError, "dsMfsEntry: Not enough room in buffer");
        return -1;
    }
    buf[0] = (dsChar)(fFileSize >> 16);
    buf[1] = (dsChar)(fFileSize & 0xFFFF);
    buf[2] = 0;
    buf += 3;
    maxlen -= 3;

    if (3 > maxlen) {
        dsLog(kLogError, "dsMfsEntry: Not enough room in buffer");
        return -1;
    }
    buf[0] = (dsChar)(fCompressedSize >> 16);
    buf[1] = (dsChar)(fCompressedSize & 0xFFFF);
    buf[2] = 0;
    buf += 3;
    maxlen -= 3;

    if (3 > maxlen) {
        dsLog(kLogError, "dsMfsEntry: Not enough room in buffer");
        return -1;
    }
    buf[0] = (dsChar)(fFlags >> 16);
    buf[1] = (dsChar)(fFlags & 0xFFFF);
    buf[2] = 0;
    buf += 3;
    maxlen -= 3;

    return (maxsave - maxlen);
}


/* dsManifest */
void dsManifest::read(const dsString& filename)
{
    size_t lineNum = 0;

    dsString path = dsConfigMgr::GetPath("DataDir", filename);
    FILE* src = fopen(path.cstr(), "r");
    if (src == NULL) {
        dsLogF(kLogWarning, "dsManifest: Could not get manifest file %s",
                            path.cstr().data());
        fStatus = kNetErrInternalError;
        return;
    }
    char buf[4096];
    while (fgets(buf, 4096, src) != NULL) {
        lineNum++;
        dsString ln(buf);
        ln = ln.trim();
        if (ln.isEmpty() || ln.left(1) == "#")
            continue;
        std::vector<dsString> mfsLine = ln.split(',');
        if (mfsLine.size() != 7) {
            dsLogF(kLogWarning, "dsManifest: Error parsing line %u of %s",
                                lineNum, filename.cstr().data());
            continue;
        }
        dsMfsEntry mfs;
        mfs.fFilename = mfsLine[0];
        mfs.fDownloadName = mfsLine[1];
        mfs.fHash = mfsLine[2];
        mfs.fCompressedHash = mfsLine[3];
        mfs.fFileSize = mfsLine[4].toUInt();
        mfs.fCompressedSize = mfsLine[5].toUInt();
        mfs.fFlags = mfsLine[6].toUInt();
        fFiles.push_back(mfs);
    }
    fclose(src);
    fStatus = kNetSuccess;
}

void dsManifest::write(dsChar* buf, size_t len)
{
    for (size_t i=0; i<getNumFiles(); i++) {
        ssize_t w = getFile(i).writeData(buf, len);
        if (w == -1) {
            fStatus = kNetErrInternalError;
            return;
        }
        buf += w;
        len -= w;
    }
    if (len != 1) {
        dsLogF(kLogError, "dsManifest: Incorrect buffer size (%u left)", len);
        fStatus = kNetErrInternalError;
        return;
    }
    *buf = 0;
}

size_t dsManifest::calcTotalSize() const
{
    size_t res = 1;     // Final Z-term
    for (size_t i=0; i<getNumFiles(); i++)
        res += getFile(i).calcSize();
    return res;
}
