#ifndef _DS_GAME_SERVER_H
#define _DS_GAME_SERVER_H

#include "../crypt/dsSha1.h"
#include "../crypt/dsRC4.h"
#include "../crypt/dsBigInteger.h"
#include "../dsUuid.h"
#include "GameMessages.h"

struct PACKED GameConnectHeader {
    dsUInt32 fHeaderSize;
    dsUuid fAccountId;
    dsUuid fAgeUuid;
};

class dsGameComm : public dsThread {
private:
    dsRC4Socket fSock;
    dsUByte fServerSeed[7], fSharedKey[7];
    class dsGameServer* fServer;

public:
    inline dsGameComm(int sock, class dsGameServer* server);
    virtual void run();
    inline virtual void finalize();

    inline void sendMsg(const msgparm_t* data, const dsNetMsg* msg);

private:
    bool calculateKeys(const dsBigInteger& Y);
};

class dsGameServer {
public:
    struct GameServerAccess {
        int fConnPipe;
        dsUuid fAgeInstance;
    };

private:
    static std::list<GameServerAccess> fServers;
    static dsMutex fServerListMutex;

    int fListenPipe;
    std::list<dsGameComm*> fClients;
    dsMutex fClientListMutex;
    dsUuid fAgeInstance;

public:
    inline dsGameServer(int pipeFd, const dsUuid& ageId);
    inline ~dsGameServer();

    static int Create(const dsUuid& ageId);
    static void AcquireClient(dsSocket* sock);

    void destroyConnection(dsGameComm* self);
    void propagateMessage(const msgparm_t* data, const dsNetMsg* msg);

private:
    void run();
    int getClientConnection();
};

#include "dsGameServer.inl"

#endif
