#include "VaultMessages.h"
#include "dsVaultNode.h"

/* Client -> Server */
static dsNetMsgField Cli2Vault_PingRequest_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Ping time
};
MAKE_NETMSG(Cli2Vault_PingRequest)

static dsNetMsgField Cli2Vault_VaultNodeCreate_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Node Data
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Cli2Vault_VaultNodeCreate)

static dsNetMsgField Cli2Vault_VaultNodeFetch_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
};
MAKE_NETMSG(Cli2Vault_VaultNodeFetch)

static dsNetMsgField Cli2Vault_VaultNodeSave_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
    { kFieldData,      16, sizeof(dsUByte)  },  // Revision ID
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Node Data
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Cli2Vault_VaultNodeSave)

static dsNetMsgField Cli2Vault_VaultNodeDelete_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
};
MAKE_NETMSG(Cli2Vault_VaultNodeDelete)

static dsNetMsgField Cli2Vault_VaultNodeAdd_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Parent ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Child ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Saver ID
};
MAKE_NETMSG(Cli2Vault_VaultNodeAdd)

static dsNetMsgField Cli2Vault_VaultNodeRemove_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Parent ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Child ID
};
MAKE_NETMSG(Cli2Vault_VaultNodeRemove)

static dsNetMsgField Cli2Vault_VaultFetchNodeRefs_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
};
MAKE_NETMSG(Cli2Vault_VaultFetchNodeRefs)

static dsNetMsgField Cli2Vault_VaultNodeFind_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Node Data
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Cli2Vault_VaultNodeFind)

static dsNetMsgField Cli2Vault_VaultSetSeen_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Parent ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Child ID
    { kFieldInteger,    0, sizeof(dsUByte)  },  // Seen
};
MAKE_NETMSG(Cli2Vault_VaultSetSeen)

static dsNetMsgField Cli2Vault_VaultSendNode_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Player ID
};
MAKE_NETMSG(Cli2Vault_VaultSendNode)


/* Server -> Client */
static dsNetMsgField Vault2Cli_PingReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Ping time
};
MAKE_NETMSG(Vault2Cli_PingReply)

static dsNetMsgField Vault2Cli_VaultNodeCreated_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
};
MAKE_NETMSG(Vault2Cli_VaultNodeCreated)

static dsNetMsgField Vault2Cli_VaultNodeFetched_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldVarCount,   0, sizeof(dsUByte)  },  // Node Data
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Vault2Cli_VaultNodeFetched)

static dsNetMsgField Vault2Cli_VaultNodeChanged_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
    { kFieldData,      16, sizeof(dsUByte)  },  // Revision Uuid
};
MAKE_NETMSG(Vault2Cli_VaultNodeChanged)

static dsNetMsgField Vault2Cli_VaultNodeDeleted_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Node ID
};
MAKE_NETMSG(Vault2Cli_VaultNodeDeleted)

static dsNetMsgField Vault2Cli_VaultNodeAdded_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Parent ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Child ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Saver ID
};
MAKE_NETMSG(Vault2Cli_VaultNodeAdded)

static dsNetMsgField Vault2Cli_VaultNodeRemoved_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Parent ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Child ID
};
MAKE_NETMSG(Vault2Cli_VaultNodeRemoved)

static dsNetMsgField Vault2Cli_VaultNodeRefsFetched_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32)         },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32)         },  // Result
    { kFieldVarCount,   0, sizeof(dsVaultNodeRef)   },  // Node Refs
    { kFieldVarPtr,     0, 0                        },
};
MAKE_NETMSG(Vault2Cli_VaultNodeRefsFetched)

static dsNetMsgField Vault2Cli_VaultNodeFindReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
    { kFieldVarCount,   0, sizeof(dsUInt32) },  // Node ID List
    { kFieldVarPtr,     0, 0                },
};
MAKE_NETMSG(Vault2Cli_VaultNodeFindReply)

static dsNetMsgField Vault2Cli_VaultSaveNodeReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Vault2Cli_VaultSaveNodeReply)

static dsNetMsgField Vault2Cli_VaultAddNodeReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Vault2Cli_VaultAddNodeReply)

static dsNetMsgField Vault2Cli_VaultRemoveNodeReply_Fields[] = {
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Trans ID
    { kFieldInteger,    0, sizeof(dsUInt32) },  // Result
};
MAKE_NETMSG(Vault2Cli_VaultRemoveNodeReply)


const dsNetMsg* GET_Cli2Vault(dsUInt32 msgId)
{
    static const dsNetMsg* s_messages[] = {
        &Cli2Vault_PingRequest,
        &Cli2Vault_VaultNodeCreate,
        &Cli2Vault_VaultNodeFetch,
        &Cli2Vault_VaultNodeSave,
        &Cli2Vault_VaultNodeDelete,
        &Cli2Vault_VaultNodeAdd,
        &Cli2Vault_VaultNodeRemove,
        &Cli2Vault_VaultFetchNodeRefs,
        &Cli2Vault_VaultNodeFind,
        &Cli2Vault_VaultSetSeen,
        &Cli2Vault_VaultSendNode,
    };
    return (msgId < COUNTOF(s_messages, dsNetMsg*) ? s_messages[msgId] : NULL);
}

const dsNetMsg* GET_Vault2Cli(dsUInt32 msgId)
{
    static const dsNetMsg* s_messages[] = {
        &Vault2Cli_PingReply,
        &Vault2Cli_VaultNodeCreated,
        &Vault2Cli_VaultNodeFetched,
        &Vault2Cli_VaultNodeChanged,
        &Vault2Cli_VaultNodeDeleted,
        &Vault2Cli_VaultNodeAdded,
        &Vault2Cli_VaultNodeRemoved,
        &Vault2Cli_VaultNodeRefsFetched,
        &Vault2Cli_VaultNodeFindReply,
        &Vault2Cli_VaultSaveNodeReply,
        &Vault2Cli_VaultAddNodeReply,
        &Vault2Cli_VaultRemoveNodeReply,
    };
    return (msgId < COUNTOF(s_messages, dsNetMsg*) ? s_messages[msgId] : NULL);
}
