#ifndef _DS_VAULT_CLIENT_H
#define _DS_VAULT_CLIENT_H

#include "../crypt/dsRC4.h"
#include "../crypt/dsBigInteger.h"
#include "VaultMessages.h"
#include "dsVaultNode.h"

class /* INTERFACE */ dsVaultMsgHandler {
public:
    virtual void onNodeCreated(const msgparm_t* msg) = 0;
    virtual void onNodeFetched(const msgparm_t* msg) = 0;
    virtual void onNodeChanged(const msgparm_t* msg) = 0;
    //virtual void onNodeDeleted(const msgparm_t* msg) = 0;
    virtual void onNodeAdded(const msgparm_t* msg) = 0;
    //virtual void onNodeRemoved(const msgparm_t* msg) = 0;
    virtual void onNodeRefsFetched(const msgparm_t* msg) = 0;
    virtual void onNodeFindReply(const msgparm_t* msg) = 0;
    virtual void onSaveNodeReply(const msgparm_t* msg) = 0;
    virtual void onAddNodeReply(const msgparm_t* msg) = 0;
    //virtual void onRemoveNodeReply(const msgparm_t* msg) = 0;
};

class dsVaultClient : public dsThread {
private:
    ENetError fStatus;
    dsRC4Socket fSock;
    dsBigInteger fClientSeed, fServerSeed;
    dsVaultMsgHandler* fCallback;

public:
    inline dsVaultClient(dsSocket* sock, dsVaultMsgHandler* callback);
    inline ~dsVaultClient();
    virtual void run();
    inline virtual void finalize();

    static dsSocket* getConnection();

    inline ENetError getStatus() const;
    inline void waitForStatus() const;

    dsUInt32 sendCreateRequest(dsUByte* nodeData, dsUInt32 nodeSize);
    dsUInt32 sendFetchRequest(dsInt32 nodeId);
    dsUInt32 sendSaveRequest(dsInt32 nodeId, const dsUuid& revisionId,
                             dsUByte* nodeData, dsUInt32 nodeSize);
    dsUInt32 sendFetchRefsRequest(dsInt32 nodeId);
    dsUInt32 sendFindRequest(dsUByte* nodeData, dsUInt32 nodeSize);
    dsUInt32 sendAddRequest(dsInt32 parent, dsInt32 child, dsInt32 saver);

    inline dsUInt32 sendCreateRequest(const dsVaultNode& node);
    inline dsUInt32 sendFindRequest(const dsVaultNode& node);

private:
    bool calculateKeys(dsUByte* y_data);
    void startCrypt(dsUByte* seed);
};

#include "dsVaultClient.inl"

#endif
