#include "dsVaultNode.h"
#include "../dsLog.h"

dsUInt32 dsVaultNode::calcSize() const
{
    dsUInt32 size = sizeof(dsUInt64);   // Field Mask

    dsUInt64 bit = 1;
    while (bit != 0 && bit <= fFieldMask) {
        switch (fFieldMask & bit) {
        case kNodeIdx:
        case kCreatorIdx:
        case kNodeType:
        case kInt32_1:
        case kInt32_2:
        case kInt32_3:
        case kInt32_4:
            size += sizeof(dsInt32);
            break;
        case kCreateTime:
        case kModifyTime:
        case kUInt32_1:
        case kUInt32_2:
        case kUInt32_3:
        case kUInt32_4:
            size += sizeof(dsUInt32);
            break;
        case kCreateAgeUuid:
        case kCreatorUuid:
        case kUuid_1:
        case kUuid_2:
        case kUuid_3:
        case kUuid_4:
            size += sizeof(dsUuid);
            break;
        case kCreateAgeName:
            size += sizeof(dsUInt32) + ((fCreateAgeName.length() + 1) * sizeof(dsChar));
            break;
        case kString64_1:
            size += sizeof(dsUInt32) + ((fString64[0].length() + 1) * sizeof(dsChar));
            break;
        case kString64_2:
            size += sizeof(dsUInt32) + ((fString64[1].length() + 1) * sizeof(dsChar));
            break;
        case kString64_3:
            size += sizeof(dsUInt32) + ((fString64[2].length() + 1) * sizeof(dsChar));
            break;
        case kString64_4:
            size += sizeof(dsUInt32) + ((fString64[3].length() + 1) * sizeof(dsChar));
            break;
        case kString64_5:
            size += sizeof(dsUInt32) + ((fString64[4].length() + 1) * sizeof(dsChar));
            break;
        case kString64_6:
            size += sizeof(dsUInt32) + ((fString64[5].length() + 1) * sizeof(dsChar));
            break;
        case kIString64_1:
            size += sizeof(dsUInt32) + ((fIString64[0].length() + 1) * sizeof(dsChar));
            break;
        case kIString64_2:
            size += sizeof(dsUInt32) + ((fIString64[1].length() + 1) * sizeof(dsChar));
            break;
        case kText_1:
            size += sizeof(dsUInt32) + ((fText[0].length() + 1) * sizeof(dsChar));
            break;
        case kText_2:
            size += sizeof(dsUInt32) + ((fText[1].length() + 1) * sizeof(dsChar));
            break;
        case kBlob_1:
            size += sizeof(dsUInt32) + fBlob[0].size();
            break;
        case kBlob_2:
            size += sizeof(dsUInt32) + fBlob[1].size();
            break;
        }
        bit <<= 1;
    }
    return size;
}

void dsVaultNode::parse(const dsUByte* buf, size_t size)
{
    if (size < sizeof(dsUInt64))
        return;
    fFieldMask = *(dsUInt64*)buf;
    buf += sizeof(dsUInt64);
    size -= sizeof(dsUInt64);

    dsUInt64 bit = 1;
    while (bit != 0 && bit <= fFieldMask && ((int)size) > 0) {
        switch (fFieldMask & bit) {
        case kNodeIdx:
            fIdx = *(dsInt32*)buf;
            buf += sizeof(dsInt32);
            size -= sizeof(dsInt32);
            break;
        case kCreateTime:
            fCreateTime = *(dsUInt32*)buf;
            buf += sizeof(dsUInt32);
            size -= sizeof(dsUInt32);
            break;
        case kModifyTime:
            fModifyTime = *(dsUInt32*)buf;
            buf += sizeof(dsUInt32);
            size -= sizeof(dsUInt32);
            break;
        case kCreateAgeName:
            {
                size_t len = *(dsUInt32*)buf / sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                fCreateAgeName.setFrom((dsChar*)buf, len-1);
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kCreateAgeUuid:
            fCreateAgeUuid = *(dsUuid*)buf;
            buf += sizeof(dsUuid);
            size -= sizeof(dsUuid);
            break;
        case kCreatorUuid:
            fCreatorUuid = *(dsUuid*)buf;
            buf += sizeof(dsUuid);
            size -= sizeof(dsUuid);
            break;
        case kCreatorIdx:
            fCreatorIdx = *(dsInt32*)buf;
            buf += sizeof(dsInt32);
            size -= sizeof(dsInt32);
            break;
        case kNodeType:
            fNodeType = *(dsInt32*)buf;
            buf += sizeof(dsInt32);
            size -= sizeof(dsInt32);
            break;
        case kInt32_1:
            fInt32[0] = *(dsInt32*)buf;
            buf += sizeof(dsInt32);
            size -= sizeof(dsInt32);
            break;
        case kInt32_2:
            fInt32[1] = *(dsInt32*)buf;
            buf += sizeof(dsInt32);
            size -= sizeof(dsInt32);
            break;
        case kInt32_3:
            fInt32[2] = *(dsInt32*)buf;
            buf += sizeof(dsInt32);
            size -= sizeof(dsInt32);
            break;
        case kInt32_4:
            fInt32[3] = *(dsInt32*)buf;
            buf += sizeof(dsInt32);
            size -= sizeof(dsInt32);
            break;
        case kUInt32_1:
            fUInt32[0] = *(dsUInt32*)buf;
            buf += sizeof(dsUInt32);
            size -= sizeof(dsUInt32);
            break;
        case kUInt32_2:
            fUInt32[1] = *(dsUInt32*)buf;
            buf += sizeof(dsUInt32);
            size -= sizeof(dsUInt32);
            break;
        case kUInt32_3:
            fUInt32[2] = *(dsUInt32*)buf;
            buf += sizeof(dsUInt32);
            size -= sizeof(dsUInt32);
            break;
        case kUInt32_4:
            fUInt32[3] = *(dsUInt32*)buf;
            buf += sizeof(dsUInt32);
            size -= sizeof(dsUInt32);
            break;
        case kUuid_1:
            fUuid[0] = *(dsUuid*)buf;
            buf += sizeof(dsUuid);
            size -= sizeof(dsUuid);
            break;
        case kUuid_2:
            fUuid[1] = *(dsUuid*)buf;
            buf += sizeof(dsUuid);
            size -= sizeof(dsUuid);
            break;
        case kUuid_3:
            fUuid[2] = *(dsUuid*)buf;
            buf += sizeof(dsUuid);
            size -= sizeof(dsUuid);
            break;
        case kUuid_4:
            fUuid[3] = *(dsUuid*)buf;
            buf += sizeof(dsUuid);
            size -= sizeof(dsUuid);
            break;
        case kString64_1:
            {
                size_t len = *(dsUInt32*)buf / sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                fString64[0].setFrom((dsChar*)buf, len-1);
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kString64_2:
            {
                size_t len = *(dsUInt32*)buf / sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                fString64[1].setFrom((dsChar*)buf, len-1);
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kString64_3:
            {
                size_t len = *(dsUInt32*)buf / sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                fString64[2].setFrom((dsChar*)buf, len-1);
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kString64_4:
            {
                size_t len = *(dsUInt32*)buf / sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                fString64[3].setFrom((dsChar*)buf, len-1);
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kString64_5:
            {
                size_t len = *(dsUInt32*)buf / sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                fString64[4].setFrom((dsChar*)buf, len-1);
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kString64_6:
            {
                size_t len = *(dsUInt32*)buf / sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                fString64[5].setFrom((dsChar*)buf, len-1);
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kIString64_1:
            {
                size_t len = *(dsUInt32*)buf / sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                fIString64[0].setFrom((dsChar*)buf, len-1);
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kIString64_2:
            {
                size_t len = *(dsUInt32*)buf / sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                fIString64[1].setFrom((dsChar*)buf, len-1);
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kText_1:
            {
                size_t len = *(dsUInt32*)buf / sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                fText[0].setFrom((dsChar*)buf, len-1);
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kText_2:
            {
                size_t len = *(dsUInt32*)buf / sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                fText[1].setFrom((dsChar*)buf, len-1);
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kBlob_1:
            {
                size_t len = *(dsUInt32*)buf;
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                fBlob[0].set(buf, len);
                buf += len;
                size -= len;
            }
            break;
        case kBlob_2:
            {
                size_t len = *(dsUInt32*)buf;
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                fBlob[1].set(buf, len);
                buf += len;
                size -= len;
            }
            break;
        }
        bit <<= 1;
    }

    if (bit <= fFieldMask || size != 0)
        dsLogF(kLogWarning, "Incomplete read of node %d", fIdx);
}

void dsVaultNode::write(dsUByte* buf, size_t size) const
{
    if (size < sizeof(dsUInt64))
        return;
    *(dsUInt64*)buf = fFieldMask;
    buf += sizeof(dsUInt64);
    size -= sizeof(dsUInt64);

    dsUInt64 bit = 1;
    while (bit != 0 && bit <= fFieldMask && ((int)size) > 0) {
        switch (fFieldMask & bit) {
        case kNodeIdx:
            *(dsInt32*)buf = fIdx;
            buf += sizeof(dsInt32);
            size -= sizeof(dsInt32);
            break;
        case kCreateTime:
            *(dsUInt32*)buf = fCreateTime;
            buf += sizeof(dsUInt32);
            size -= sizeof(dsUInt32);
            break;
        case kModifyTime:
            *(dsUInt32*)buf = fModifyTime;
            buf += sizeof(dsUInt32);
            size -= sizeof(dsUInt32);
            break;
        case kCreateAgeName:
            {
                size_t len = fCreateAgeName.length() + 1;
                *(dsUInt32*)buf = len * sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                memcpy(buf, fCreateAgeName.data(), len * sizeof(dsChar));
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kCreateAgeUuid:
            *(dsUuid*)buf = fCreateAgeUuid;
            buf += sizeof(dsUuid);
            size -= sizeof(dsUuid);
            break;
        case kCreatorUuid:
            *(dsUuid*)buf = fCreatorUuid;
            buf += sizeof(dsUuid);
            size -= sizeof(dsUuid);
            break;
        case kCreatorIdx:
            *(dsInt32*)buf = fCreatorIdx;
            buf += sizeof(dsInt32);
            size -= sizeof(dsInt32);
            break;
        case kNodeType:
            *(dsInt32*)buf = fNodeType;
            buf += sizeof(dsInt32);
            size -= sizeof(dsInt32);
            break;
        case kInt32_1:
            *(dsInt32*)buf = fInt32[0];
            buf += sizeof(dsInt32);
            size -= sizeof(dsInt32);
            break;
        case kInt32_2:
            *(dsInt32*)buf = fInt32[1];
            buf += sizeof(dsInt32);
            size -= sizeof(dsInt32);
            break;
        case kInt32_3:
            *(dsInt32*)buf = fInt32[2];
            buf += sizeof(dsInt32);
            size -= sizeof(dsInt32);
            break;
        case kInt32_4:
            *(dsInt32*)buf = fInt32[3];
            buf += sizeof(dsInt32);
            size -= sizeof(dsInt32);
            break;
        case kUInt32_1:
            *(dsUInt32*)buf = fUInt32[0];
            buf += sizeof(dsUInt32);
            size -= sizeof(dsUInt32);
            break;
        case kUInt32_2:
            *(dsUInt32*)buf = fUInt32[1];
            buf += sizeof(dsUInt32);
            size -= sizeof(dsUInt32);
            break;
        case kUInt32_3:
            *(dsUInt32*)buf = fUInt32[2];
            buf += sizeof(dsUInt32);
            size -= sizeof(dsUInt32);
            break;
        case kUInt32_4:
            *(dsUInt32*)buf = fUInt32[3];
            buf += sizeof(dsUInt32);
            size -= sizeof(dsUInt32);
            break;
        case kUuid_1:
            *(dsUuid*)buf = fUuid[0];
            buf += sizeof(dsUuid);
            size -= sizeof(dsUuid);
            break;
        case kUuid_2:
            *(dsUuid*)buf = fUuid[1];
            buf += sizeof(dsUuid);
            size -= sizeof(dsUuid);
            break;
        case kUuid_3:
            *(dsUuid*)buf = fUuid[2];
            buf += sizeof(dsUuid);
            size -= sizeof(dsUuid);
            break;
        case kUuid_4:
            *(dsUuid*)buf = fUuid[3];
            buf += sizeof(dsUuid);
            size -= sizeof(dsUuid);
            break;
        case kString64_1:
            {
                size_t len = fString64[0].length() + 1;
                *(dsUInt32*)buf = len * sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                memcpy(buf, fString64[0].data(), len * sizeof(dsChar));
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kString64_2:
            {
                size_t len = fString64[1].length() + 1;
                *(dsUInt32*)buf = len * sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                memcpy(buf, fString64[1].data(), len * sizeof(dsChar));
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kString64_3:
            {
                size_t len = fString64[2].length() + 1;
                *(dsUInt32*)buf = len * sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                memcpy(buf, fString64[2].data(), len * sizeof(dsChar));
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kString64_4:
            {
                size_t len = fString64[3].length() + 1;
                *(dsUInt32*)buf = len * sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                memcpy(buf, fString64[3].data(), len * sizeof(dsChar));
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kString64_5:
            {
                size_t len = fString64[4].length() + 1;
                *(dsUInt32*)buf = len * sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                memcpy(buf, fString64[4].data(), len * sizeof(dsChar));
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kString64_6:
            {
                size_t len = fString64[5].length() + 1;
                *(dsUInt32*)buf = len * sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                memcpy(buf, fString64[5].data(), len * sizeof(dsChar));
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kIString64_1:
            {
                size_t len = fIString64[0].length() + 1;
                *(dsUInt32*)buf = len * sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                memcpy(buf, fIString64[0].data(), len * sizeof(dsChar));
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kIString64_2:
            {
                size_t len = fIString64[1].length() + 1;
                *(dsUInt32*)buf = len * sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                memcpy(buf, fIString64[1].data(), len * sizeof(dsChar));
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kText_1:
            {
                size_t len = fText[0].length() + 1;
                *(dsUInt32*)buf = len * sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                memcpy(buf, fText[0].data(), len * sizeof(dsChar));
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kText_2:
            {
                size_t len = fText[1].length() + 1;
                *(dsUInt32*)buf = len * sizeof(dsChar);
                buf += sizeof(dsUInt32);
                size -= sizeof(dsUInt32);

                memcpy(buf, fText[1].data(), len * sizeof(dsChar));
                buf += len * sizeof(dsChar);
                size -= len * sizeof(dsChar);
            }
            break;
        case kBlob_1:
            *(dsUInt32*)buf = fBlob[0].size();
            buf += sizeof(dsUInt32);
            size -= sizeof(dsUInt32);

            memcpy(buf, fBlob[0].data(), fBlob[0].size());
            buf += fBlob[0].size();
            size -= fBlob[0].size();
            break;
        case kBlob_2:
            *(dsUInt32*)buf = fBlob[1].size();
            buf += sizeof(dsUInt32);
            size -= sizeof(dsUInt32);

            memcpy(buf, fBlob[1].data(), fBlob[1].size());
            buf += fBlob[1].size();
            size -= fBlob[1].size();
            break;
        }
        bit <<= 1;
    }

    if (bit <= fFieldMask || size != 0)
        dsLogF(kLogWarning, "Node %d is truncated", fIdx);
}

void dsVaultNode::fromDbRow(const dsPostgresRow& row)
{
    clearAll();
    setIdx(row["idx"].getInt());
    setNodeType(row["\"NodeType\""].getInt());
    if (!row["\"CreateTime\""].isNull())
        setCreateTime(row["\"CreateTime\""].getUInt());
    if (!row["\"ModifyTime\""].isNull())
        setModifyTime(row["\"ModifyTime\""].getUInt());
    if (!row["\"CreateAgeName\""].isNull())
        setCreateAgeName(row["\"CreateAgeName\""].getString());
    if (!row["\"CreateAgeUuid\""].isNull())
        setCreateAgeUuid(dsUuid::fromString(row["\"CreateAgeUuid\""].getString()));
    if (!row["\"CreatorUuid\""].isNull())
        setCreatorUuid(dsUuid::fromString(row["\"CreatorUuid\""].getString()));
    if (!row["\"CreatorIdx\""].isNull())
        setCreatorIdx(row["\"CreatorIdx\""].getInt());
    if (!row["\"Int32_1\""].isNull())
        setInt32(0, row["\"Int32_1\""].getInt());
    if (!row["\"Int32_2\""].isNull())
        setInt32(1, row["\"Int32_2\""].getInt());
    if (!row["\"Int32_3\""].isNull())
        setInt32(2, row["\"Int32_3\""].getInt());
    if (!row["\"Int32_4\""].isNull())
        setInt32(3, row["\"Int32_4\""].getInt());
    if (!row["\"UInt32_1\""].isNull())
        setUInt32(0, row["\"UInt32_1\""].getUInt());
    if (!row["\"UInt32_2\""].isNull())
        setUInt32(1, row["\"UInt32_2\""].getUInt());
    if (!row["\"UInt32_3\""].isNull())
        setUInt32(2, row["\"UInt32_3\""].getUInt());
    if (!row["\"UInt32_4\""].isNull())
        setUInt32(3, row["\"UInt32_4\""].getUInt());
    if (!row["\"Uuid_1\""].isNull())
        setUuid(0, dsUuid::fromString(row["\"Uuid_1\""].getString()));
    if (!row["\"Uuid_2\""].isNull())
        setUuid(1, dsUuid::fromString(row["\"Uuid_2\""].getString()));
    if (!row["\"Uuid_3\""].isNull())
        setUuid(2, dsUuid::fromString(row["\"Uuid_3\""].getString()));
    if (!row["\"Uuid_4\""].isNull())
        setUuid(3, dsUuid::fromString(row["\"Uuid_4\""].getString()));
    if (!row["\"String64_1\""].isNull())
        setString64(0, row["\"String64_1\""].getString());
    if (!row["\"String64_2\""].isNull())
        setString64(1, row["\"String64_2\""].getString());
    if (!row["\"String64_3\""].isNull())
        setString64(2, row["\"String64_3\""].getString());
    if (!row["\"String64_4\""].isNull())
        setString64(3, row["\"String64_4\""].getString());
    if (!row["\"String64_5\""].isNull())
        setString64(4, row["\"String64_5\""].getString());
    if (!row["\"String64_6\""].isNull())
        setString64(5, row["\"String64_6\""].getString());
    if (!row["\"IString64_1\""].isNull())
        setIString64(0, row["\"IString64_1\""].getString());
    if (!row["\"IString64_2\""].isNull())
        setIString64(1, row["\"IString64_2\""].getString());
    if (!row["\"Text_1\""].isNull())
        setText(0, row["\"Text_1\""].getString());
    if (!row["\"Text_2\""].isNull())
        setText(1, row["\"Text_2\""].getString());
    if (!row["\"Blob_1\""].isNull())
        setBlob(0, row["\"Blob_1\""].getBlob());
    if (!row["\"Blob_2\""].isNull())
        setBlob(1, row["\"Blob_2\""].getBlob());
}

dsString dsVaultNode::queryStrFields() const
{
    dsString result;

    dsUInt64 bit = 1;
    while (bit != 0 && bit <= fFieldMask) {
        if (!result.isEmpty() && ((bit & fFieldMask) != 0))
            result += ", ";

        switch (fFieldMask & bit) {
        case kCreateTime:
            result += "\"CreateTime\"";
            break;
        case kModifyTime:
            result += "\"ModifyTime\"";
            break;
        case kCreateAgeName:
            result += "\"CreateAgeName\"";
            break;
        case kCreateAgeUuid:
            result += "\"CreateAgeUuid\"";
            break;
        case kCreatorUuid:
            result += "\"CreatorUuid\"";
            break;
        case kCreatorIdx:
            result += "\"CreatorIdx\"";
            break;
        case kNodeType:
            result += "\"NodeType\"";
            break;
        case kInt32_1:
            result += "\"Int32_1\"";
            break;
        case kInt32_2:
            result += "\"Int32_2\"";
            break;
        case kInt32_3:
            result += "\"Int32_3\"";
            break;
        case kInt32_4:
            result += "\"Int32_4\"";
            break;
        case kUInt32_1:
            result += "\"UInt32_1\"";
            break;
        case kUInt32_2:
            result += "\"UInt32_2\"";
            break;
        case kUInt32_3:
            result += "\"UInt32_3\"";
            break;
        case kUInt32_4:
            result += "\"UInt32_4\"";
            break;
        case kUuid_1:
            result += "\"Uuid_1\"";
            break;
        case kUuid_2:
            result += "\"Uuid_2\"";
            break;
        case kUuid_3:
            result += "\"Uuid_3\"";
            break;
        case kUuid_4:
            result += "\"Uuid_4\"";
            break;
        case kString64_1:
            result += "\"String64_1\"";
            break;
        case kString64_2:
            result += "\"String64_2\"";
            break;
        case kString64_3:
            result += "\"String64_3\"";
            break;
        case kString64_4:
            result += "\"String64_4\"";
            break;
        case kString64_5:
            result += "\"String64_5\"";
            break;
        case kString64_6:
            result += "\"String64_6\"";
            break;
        case kIString64_1:
            result += "\"IString64_1\"";
            break;
        case kIString64_2:
            result += "\"IString64_2\"";
            break;
        case kText_1:
            result += "\"Text_1\"";
            break;
        case kText_2:
            result += "\"Text_2\"";
            break;
        case kBlob_1:
            result += "\"Blob_1\"";
            break;
        case kBlob_2:
            result += "\"Blob_2\"";
            break;
        }
        bit <<= 1;
    }
    return result;
}

dsString dsVaultNode::queryStrValues(dsPostgres* db) const
{
    dsString result;

    char buf[256];

    dsUInt64 bit = 1;
    while (bit != 0 && bit <= fFieldMask) {
        if (!result.isEmpty() && ((bit & fFieldMask) != 0))
            result += ", ";

        switch (fFieldMask & bit) {
        case kCreateTime:
            result += dsString::Format("%d", (dsInt32)fCreateTime);
            break;
        case kModifyTime:
            result += dsString::Format("%d", (dsInt32)fModifyTime);
            break;
        case kCreateAgeName:
            result += "'" + fCreateAgeName + "'";
            break;
        case kCreateAgeUuid:
            result += dsString::Format("'%s'", fCreateAgeUuid.asString().cstr().data());
            break;
        case kCreatorUuid:
            result += dsString::Format("'%s'", fCreatorUuid.asString().cstr().data());
            break;
        case kCreatorIdx:
            result += dsString::Format("%d", fCreatorIdx);
            break;
        case kNodeType:
            result += dsString::Format("%d", fNodeType);
            break;
        case kInt32_1:
            result += dsString::Format("%d", fInt32[0]);
            break;
        case kInt32_2:
            result += dsString::Format("%d", fInt32[1]);
            break;
        case kInt32_3:
            result += dsString::Format("%d", fInt32[2]);
            break;
        case kInt32_4:
            result += dsString::Format("%d", fInt32[3]);
            break;
        case kUInt32_1:
            result += dsString::Format("%d", (dsInt32)fUInt32[0]);
            break;
        case kUInt32_2:
            result += dsString::Format("%d", (dsInt32)fUInt32[1]);
            break;
        case kUInt32_3:
            result += dsString::Format("%d", (dsInt32)fUInt32[2]);
            break;
        case kUInt32_4:
            result += dsString::Format("%d", (dsInt32)fUInt32[3]);
            break;
        case kUuid_1:
            result += dsString::Format("'%s'", fUuid[0].asString().cstr().data());
            break;
        case kUuid_2:
            result += dsString::Format("'%s'", fUuid[1].asString().cstr().data());
            break;
        case kUuid_3:
            result += dsString::Format("'%s'", fUuid[2].asString().cstr().data());
            break;
        case kUuid_4:
            result += dsString::Format("'%s'", fUuid[3].asString().cstr().data());
            break;
        case kString64_1:
            db->escapeString(buf, fString64[0].cstr(), 256);
            result += dsString::Format("'%s'", buf);
            break;
        case kString64_2:
            db->escapeString(buf, fString64[1].cstr(), 256);
            result += dsString::Format("'%s'", buf);
            break;
        case kString64_3:
            db->escapeString(buf, fString64[2].cstr(), 256);
            result += dsString::Format("'%s'", buf);
            break;
        case kString64_4:
            db->escapeString(buf, fString64[3].cstr(), 256);
            result += dsString::Format("'%s'", buf);
            break;
        case kString64_5:
            db->escapeString(buf, fString64[4].cstr(), 256);
            result += dsString::Format("'%s'", buf);
            break;
        case kString64_6:
            db->escapeString(buf, fString64[5].cstr(), 256);
            result += dsString::Format("'%s'", buf);
            break;
        case kIString64_1:
            db->escapeString(buf, fIString64[0].cstr(), 256);
            result += dsString::Format("'%s'", buf);
            break;
        case kIString64_2:
            db->escapeString(buf, fIString64[0].cstr(), 256);
            result += dsString::Format("'%s'", buf);
            break;
        case kText_1:
            db->escapeString(buf, fText[0].cstr(), 256);
            result += dsString::Format("'%s'", buf);
            break;
        case kText_2:
            db->escapeString(buf, fText[1].cstr(), 256);
            result += dsString::Format("'%s'", buf);
            break;
        case kBlob_1:
            result += "'" + Base64(fBlob[0].data(), fBlob[0].size()) + "'";
            break;
        case kBlob_2:
            result += "'" + Base64(fBlob[1].data(), fBlob[1].size()) + "'";
            break;
        }
        bit <<= 1;
    }
    return result;
}

dsString dsVaultNode::queryStrSelect(dsPostgres* db) const
{
    dsString result;
    char buf[256];

    dsUInt64 bit = 1;
    while (bit != 0 && bit <= fFieldMask) {
        if (!result.isEmpty() && ((bit & fFieldMask) != 0))
            result += " AND ";

        switch (fFieldMask & bit) {
        //case kCreateTime:
        //    result += dsString::Format("\"CreateTime\"=%d", (dsInt32)fCreateTime);
        //    break;
        //case kModifyTime:
        //    result += dsString::Format("\"ModifyTime\"=%d", (dsInt32)fModifyTime);
        //    break;
        case kCreateAgeName:
            result += "LOWER(\"CreateAgeName\")=LOWER('" + fCreateAgeName + "')";
            break;
        case kCreateAgeUuid:
            result += dsString::Format("\"CreateAgeUuid\"='%s'",
                                       fCreateAgeUuid.asString().cstr().data());
            break;
        case kCreatorUuid:
            result += dsString::Format("\"CreatorUuid\"='%s'",
                                       fCreatorUuid.asString().cstr().data());
            break;
        case kCreatorIdx:
            result += dsString::Format("\"CreatorIdx\"=%d", fCreatorIdx);
            break;
        case kNodeType:
            result += dsString::Format("\"NodeType\"=%d", fNodeType);
            break;
        case kInt32_1:
            result += dsString::Format("\"Int32_1\"=%d", fInt32[0]);
            break;
        case kInt32_2:
            result += dsString::Format("\"Int32_2\"=%d", fInt32[1]);
            break;
        case kInt32_3:
            result += dsString::Format("\"Int32_3\"=%d", fInt32[2]);
            break;
        case kInt32_4:
            result += dsString::Format("\"Int32_4\"=%d", fInt32[3]);
            break;
        case kUInt32_1:
            result += dsString::Format("\"UInt32_1\"=%d", (dsInt32)fUInt32[0]);
            break;
        case kUInt32_2:
            result += dsString::Format("\"UInt32_2\"=%d", (dsInt32)fUInt32[1]);
            break;
        case kUInt32_3:
            result += dsString::Format("\"UInt32_3\"=%d", (dsInt32)fUInt32[2]);
            break;
        case kUInt32_4:
            result += dsString::Format("\"UInt32_4\"=%d", (dsInt32)fUInt32[3]);
            break;
        case kUuid_1:
            result += dsString::Format("\"Uuid_1\"='%s'",
                                       fUuid[0].asString().cstr().data());
            break;
        case kUuid_2:
            result += dsString::Format("\"Uuid_2\"='%s'",
                                       fUuid[1].asString().cstr().data());
            break;
        case kUuid_3:
            result += dsString::Format("\"Uuid_3\"='%s'",
                                       fUuid[2].asString().cstr().data());
            break;
        case kUuid_4:
            result += dsString::Format("\"Uuid_4\"='%s'",
                                       fUuid[3].asString().cstr().data());
            break;
        case kString64_1:
            db->escapeString(buf, fString64[0].cstr(), 256);
            result += dsString::Format("\"String64_1\"='%s'", buf);
            break;
        case kString64_2:
            db->escapeString(buf, fString64[1].cstr(), 256);
            result += dsString::Format("\"String64_2\"='%s'", buf);
            break;
        case kString64_3:
            db->escapeString(buf, fString64[2].cstr(), 256);
            result += dsString::Format("\"String64_3\"='%s'", buf);
            break;
        case kString64_4:
            db->escapeString(buf, fString64[3].cstr(), 256);
            result += dsString::Format("\"String64_4\"='%s'", buf);
            break;
        case kString64_5:
            db->escapeString(buf, fString64[4].cstr(), 256);
            result += dsString::Format("\"String64_5\"='%s'", buf);
            break;
        case kString64_6:
            db->escapeString(buf, fString64[5].cstr(), 256);
            result += dsString::Format("\"String64_6\"='%s'", buf);
            break;
        case kIString64_1:
            db->escapeString(buf, fIString64[0].cstr(), 256);
            result += dsString::Format("LOWER(\"IString64_1\")=LOWER('%s')", buf);
            break;
        case kIString64_2:
            db->escapeString(buf, fIString64[1].cstr(), 256);
            result += dsString::Format("LOWER(\"IString64_2\")=LOWER('%s')", buf);
            break;
        case kText_1:
            db->escapeString(buf, fText[0].cstr(), 256);
            result += dsString::Format("\"Text_1\"='%s'", buf);
            break;
        case kText_2:
            db->escapeString(buf, fText[1].cstr(), 256);
            result += dsString::Format("\"Text_2\"='%s'", buf);
            break;
        case kBlob_1:
            result += "\"Blob_1\"='" + Base64(fBlob[0].data(), fBlob[0].size()) + "'";
            break;
        case kBlob_2:
            result += "\"Blob_2\"='" + Base64(fBlob[1].data(), fBlob[1].size()) + "'";
            break;
        }
        bit <<= 1;
    }
    return result;
}

dsString dsVaultNode::queryStrUpdate(dsPostgres* db) const
{
    dsString result;
    char buf[256];

    dsUInt64 bit = 1;
    while (bit != 0 && bit <= fFieldMask) {
        if (!result.isEmpty() && ((bit & fFieldMask) != 0))
            result += ", ";

        switch (fFieldMask & bit) {
        case kCreateTime:
            result += dsString::Format("\"CreateTime\"=%d", (dsInt32)fCreateTime);
            break;
        case kModifyTime:
            result += dsString::Format("\"ModifyTime\"=%d", (dsInt32)fModifyTime);
            break;
        case kCreateAgeName:
            result += "\"CreateAgeName\"='" + fCreateAgeName + "'";
            break;
        case kCreateAgeUuid:
            result += dsString::Format("\"CreateAgeUuid\"='%s'",
                                       fCreateAgeUuid.asString().cstr().data());
            break;
        case kCreatorUuid:
            result += dsString::Format("\"CreatorUuid\"='%s'",
                                       fCreatorUuid.asString().cstr().data());
            break;
        case kCreatorIdx:
            result += dsString::Format("\"CreatorIdx\"=%d", fCreatorIdx);
            break;
        case kNodeType:
            result += dsString::Format("\"NodeType\"=%d", fNodeType);
            break;
        case kInt32_1:
            result += dsString::Format("\"Int32_1\"=%d", fInt32[0]);
            break;
        case kInt32_2:
            result += dsString::Format("\"Int32_2\"=%d", fInt32[1]);
            break;
        case kInt32_3:
            result += dsString::Format("\"Int32_3\"=%d", fInt32[2]);
            break;
        case kInt32_4:
            result += dsString::Format("\"Int32_4\"=%d", fInt32[3]);
            break;
        case kUInt32_1:
            result += dsString::Format("\"UInt32_1\"=%d", (dsInt32)fUInt32[0]);
            break;
        case kUInt32_2:
            result += dsString::Format("\"UInt32_2\"=%d", (dsInt32)fUInt32[1]);
            break;
        case kUInt32_3:
            result += dsString::Format("\"UInt32_3\"=%d", (dsInt32)fUInt32[2]);
            break;
        case kUInt32_4:
            result += dsString::Format("\"UInt32_4\"=%d", (dsInt32)fUInt32[3]);
            break;
        case kUuid_1:
            result += dsString::Format("\"Uuid_1\"='%s'",
                                       fUuid[0].asString().cstr().data());
            break;
        case kUuid_2:
            result += dsString::Format("\"Uuid_2\"='%s'",
                                       fUuid[1].asString().cstr().data());
            break;
        case kUuid_3:
            result += dsString::Format("\"Uuid_3\"='%s'",
                                       fUuid[2].asString().cstr().data());
            break;
        case kUuid_4:
            result += dsString::Format("\"Uuid_4\"='%s'",
                                       fUuid[3].asString().cstr().data());
            break;
        case kString64_1:
            db->escapeString(buf, fString64[0].cstr(), 256);
            result += dsString::Format("\"String64_1\"='%s'", buf);
            break;
        case kString64_2:
            db->escapeString(buf, fString64[1].cstr(), 256);
            result += dsString::Format("\"String64_2\"='%s'", buf);
            break;
        case kString64_3:
            db->escapeString(buf, fString64[2].cstr(), 256);
            result += dsString::Format("\"String64_3\"='%s'", buf);
            break;
        case kString64_4:
            db->escapeString(buf, fString64[3].cstr(), 256);
            result += dsString::Format("\"String64_4\"='%s'", buf);
            break;
        case kString64_5:
            db->escapeString(buf, fString64[4].cstr(), 256);
            result += dsString::Format("\"String64_5\"='%s'", buf);
            break;
        case kString64_6:
            db->escapeString(buf, fString64[5].cstr(), 256);
            result += dsString::Format("\"String64_6\"='%s'", buf);
            break;
        case kIString64_1:
            db->escapeString(buf, fIString64[0].cstr(), 256);
            result += dsString::Format("\"IString64_1\"='%s'", buf);
            break;
        case kIString64_2:
            db->escapeString(buf, fIString64[1].cstr(), 256);
            result += dsString::Format("\"IString64_2\"='%s'", buf);
            break;
        case kText_1:
            db->escapeString(buf, fText[0].cstr(), 256);
            result += dsString::Format("\"Text_1\"='%s'", buf);
            break;
        case kText_2:
            db->escapeString(buf, fText[1].cstr(), 256);
            result += dsString::Format("\"Text_2\"='%s'", buf);
            break;
        case kBlob_1:
            result += "\"Blob_1\"='" + Base64(fBlob[0].data(), fBlob[0].size()) + "'";
            break;
        case kBlob_2:
            result += "\"Blob_2\"='" + Base64(fBlob[1].data(), fBlob[1].size()) + "'";
            break;
        }
        bit <<= 1;
    }
    return result;
}

void dsVaultNode::setIdx(dsInt32 idx)
{
    fIdx = idx;
}

void dsVaultNode::setCreateTime(dsUInt32 createTime)
{
    fCreateTime = createTime;
    fFieldMask |= (dsUInt64)kCreateTime;
}

void dsVaultNode::setModifyTime(dsUInt32 modTime)
{
    fModifyTime = modTime;
    fFieldMask |= (dsUInt64)kModifyTime;
}

void dsVaultNode::setCreateAgeName(const dsString& name)
{
    fCreateAgeName = name;
    fFieldMask |= (dsUInt64)kCreateAgeName;
}

void dsVaultNode::setCreateAgeUuid(const dsUuid& uuid)
{
    fCreateAgeUuid = uuid;
    fFieldMask |= (dsUInt64)kCreateAgeUuid;
}

void dsVaultNode::setCreatorUuid(const dsUuid& uuid)
{
    fCreatorUuid = uuid;
    fFieldMask |= (dsUInt64)kCreatorUuid;
}

void dsVaultNode::setCreatorIdx(dsInt32 idx)
{
    fCreatorIdx = idx;
    fFieldMask |= (dsUInt64)kCreatorIdx;
}

void dsVaultNode::setNodeType(dsInt32 type)
{
    fNodeType = type;
}

void dsVaultNode::setInt32(int which, dsInt32 value)
{
    fInt32[which] = value;
    fFieldMask |= (dsUInt64)(kInt32_1 << which);
}

void dsVaultNode::setUInt32(int which, dsUInt32 value)
{
    fUInt32[which] = value;
    fFieldMask |= (dsUInt64)(kUInt32_1 << which);
}

void dsVaultNode::setUuid(int which, const dsUuid& value)
{
    fUuid[which] = value;
    fFieldMask |= (dsUInt64)(kUuid_1 << which);
}

void dsVaultNode::setString64(int which, const dsString& value)
{
    fString64[which] = value;
    fFieldMask |= (dsUInt64)(kString64_1 << which);
}

void dsVaultNode::setIString64(int which, const dsString& value)
{
    fIString64[which] = value;
    fFieldMask |= (dsUInt64)(kIString64_1 << which);
}

void dsVaultNode::setText(int which, const dsString& value)
{
    fText[which] = value;
    fFieldMask |= (dsUInt64)(kText_1 << which);
}

void dsVaultNode::setBlob(int which, const dsBlob& value)
{
    fBlob[which] = value;
    fFieldMask |= (dsUInt64)(kBlob_1 << which);
}

void dsVaultNode::clearIdx()
{
    fIdx = 0;
}

void dsVaultNode::clearCreateTime()
{
    fFieldMask &= ~(dsUInt64)kCreateTime;
}

void dsVaultNode::clearModifyTime()
{
    fFieldMask &= ~(dsUInt64)kModifyTime;
}

void dsVaultNode::clearCreateAgeName()
{
    fFieldMask &= ~(dsUInt64)kCreateAgeName;
}

void dsVaultNode::clearCreateAgeUuid()
{
    fFieldMask &= ~(dsUInt64)kCreateAgeUuid;
}

void dsVaultNode::clearCreatorUuid()
{
    fFieldMask &= ~(dsUInt64)kCreatorUuid;
}

void dsVaultNode::clearCreatorIdx()
{
    fFieldMask &= ~(dsUInt64)kCreatorIdx;
}

void dsVaultNode::clearNodeType()
{
    fNodeType = kNodeInvalid;
}

void dsVaultNode::clearInt32(int which)
{
    fFieldMask &= ~(dsUInt64)(kInt32_1 << which);
}

void dsVaultNode::clearUInt32(int which)
{
    fFieldMask &= ~(dsUInt64)(kUInt32_1 << which);
}

void dsVaultNode::clearUuid(int which)
{
    fFieldMask &= ~(dsUInt64)(kUuid_1 << which);
}

void dsVaultNode::clearString64(int which)
{
    fFieldMask &= ~(dsUInt64)(kString64_1 << which);
}

void dsVaultNode::clearIString64(int which)
{
    fFieldMask &= ~(dsUInt64)(kIString64_1 << which);
}

void dsVaultNode::clearText(int which)
{
    fFieldMask &= ~(dsUInt64)(kText_1 << which);
}

void dsVaultNode::clearBlob(int which)
{
    fFieldMask &= ~(dsUInt64)(kBlob_1 << which);
}
