#ifndef _DS_VAULT_NODE_H
#define _DS_VAULT_NODE_H

#include "../db/dsPostgres.h"

enum ENodeType {
    kNodeInvalid, kNodeVNodeMgrLow, kNodePlayer, kNodeAge, kNodeGameServer,
    kNodeAdmin, kNodeVaultServer, kNodeCCR, kNodeVNodeMgrHigh = 21,
    kNodeFolder, kNodePlayerInfo, kNodeSystem, kNodeImage, kNodeTextNote,
    kNodeSDL, kNodeAgeLink, kNodeChronicle, kNodePlayerInfoList,
    kNodeUNUSED, kNodeMarker, kNodeAgeInfo, kNodeAgeInfoList,
    kNodeMarkerList
};

enum EStandardNodes {
    kUserDefinedNode, kInboxFolder, kBuddyListFolder, kIgnoreListFolder,
    kPeopleIKnowAboutFolder, kVaultMgrGlobalDataFolder, kChronicleFolder,
    kAvatarOutfitFolder, kAgeTypeJournalFolder, kSubAgesFolder,
    kDeviceInboxFolder, kHoodMembersFolder, kAllPlayersFolder,
    kAgeMembersFolder, kAgeJournalsFolder, kAgeDevicesFolder,
    kAgeInstanceSDLNode, kAgeGlobalSDLNode, kCanVisitFolder, kAgeOwnersFolder,
    kAllAgeGlobalSDLNodesFolder, kPlayerInfoNode, kPublicAgesFolder,
    kAgesIOwnFolder, kAgesICanVisitFolder, kAvatarClosetFolder, kAgeInfoNode,
    kSystemNode, kPlayerInviteFolder, kCCRPlayersFolder, kGlobalInboxFolder,
    kChildAgesFolder, kGameScoresFolder, kLastStandardNode
};

enum ENoteTypes {
    kNoteGeneric, kNoteCCRPetition, kNoteDevice, kNoteInvite, kNoteVisit,
    kNoteUnVisit, kNumNoteTypes
};

enum EImageTypes {
    kImgNone, kImgJPEG
};

enum {
    kNodeIdx        = (1<<0),
    kCreateTime     = (1<<1),
    kModifyTime     = (1<<2),
    kCreateAgeName  = (1<<3),
    kCreateAgeUuid  = (1<<4),
    kCreatorUuid    = (1<<5),
    kCreatorIdx     = (1<<6),
    kNodeType       = (1<<7),
    kInt32_1        = (1<<8),
    kInt32_2        = (1<<9),
    kInt32_3        = (1<<10),
    kInt32_4        = (1<<11),
    kUInt32_1       = (1<<12),
    kUInt32_2       = (1<<13),
    kUInt32_3       = (1<<14),
    kUInt32_4       = (1<<15),
    kUuid_1         = (1<<16),
    kUuid_2         = (1<<17),
    kUuid_3         = (1<<18),
    kUuid_4         = (1<<19),
    kString64_1     = (1<<20),
    kString64_2     = (1<<21),
    kString64_3     = (1<<22),
    kString64_4     = (1<<23),
    kString64_5     = (1<<24),
    kString64_6     = (1<<25),
    kIString64_1    = (1<<26),
    kIString64_2    = (1<<27),
    kText_1         = (1<<28),
    kText_2         = (1<<29),
    kBlob_1         = (1<<30),
    kBlob_2         = (1<<31)
};

class dsVaultNode {
private:
    dsUInt64 fFieldMask;

public:
    dsInt32 fIdx;
    dsUInt32 fCreateTime, fModifyTime;
    dsString fCreateAgeName;
    dsUuid fCreateAgeUuid, fCreatorUuid;
    dsInt32 fCreatorIdx, fNodeType;
    dsInt32 fInt32[4];
    dsUInt32 fUInt32[4];
    dsUuid fUuid[4];
    dsString fString64[6], fIString64[2];
    dsString fText[2];
    dsBlob fBlob[2];

public:
    inline dsVaultNode();

    dsUInt32 calcSize() const;
    void parse(const dsUByte* buf, size_t size);
    void write(dsUByte* buf, size_t size) const;
    void fromDbRow(const dsPostgresRow& row);
    dsString queryStrFields() const;
    dsString queryStrValues(dsPostgres* db) const;
    dsString queryStrSelect(dsPostgres* db) const;
    dsString queryStrUpdate(dsPostgres* db) const;

    inline void setTimeNow();
    inline void modifyNow();

    void setIdx(dsInt32 idx);
    void setCreateTime(dsUInt32 createTime);
    void setModifyTime(dsUInt32 modTime);
    void setCreateAgeName(const dsString& name);
    void setCreateAgeUuid(const dsUuid& uuid);
    void setCreatorUuid(const dsUuid& uuid);
    void setCreatorIdx(dsInt32 idx);
    void setNodeType(dsInt32 type);
    void setInt32(int which, dsInt32 value);
    void setUInt32(int which, dsUInt32 value);
    void setUuid(int which, const dsUuid& value);
    void setString64(int which, const dsString& value);
    void setIString64(int which, const dsString& value);
    void setText(int which, const dsString& value);
    void setBlob(int which, const dsBlob& value);

    void clearIdx();
    void clearCreateTime();
    void clearModifyTime();
    void clearCreateAgeName();
    void clearCreateAgeUuid();
    void clearCreatorUuid();
    void clearCreatorIdx();
    void clearNodeType();
    void clearInt32(int which);
    void clearUInt32(int which);
    void clearUuid(int which);
    void clearString64(int which);
    void clearIString64(int which);
    void clearText(int which);
    void clearBlob(int which);
    inline void clearAll();
};

struct PACKED dsVaultNodeRef {
    dsUInt32 fParentIdx, fChildIdx, fOwnerIdx;
    bool fSeen;
};

#include "dsVaultNode.inl"

#endif
