#ifndef _DS_VAULT_SERVER_H
#define _DS_VAULT_SERVER_H

#include "../crypt/dsSha1.h"
#include "../db/dsPostgres.h"
#include "../dsUuid.h"
#include "dsVaultClient.h"

struct PACKED VaultConnectHeader {
    dsUInt32 fHeaderSize;
    dsUuid fToken;
};

class dsVaultComm : public dsThread {
private:
    dsRC4Socket fSock;
    dsUByte fServerSeed[7], fSharedKey[7];
    dsPostgres* fVaultDb;

public:
    inline dsVaultComm(int sock, dsPostgres* vaultDb);
    virtual void run();
    inline virtual void finalize();

    inline void sendMsg(const msgparm_t* data, const dsNetMsg* msg);

private:
    bool calculateKeys(const dsBigInteger& Y);
};

class dsVaultServer {
private:
    static std::list<dsVaultComm*> fClients;
    static dsMutex fClientListMutex;
    dsPostgres fVaultDb;
    static dsInt32 sRootIdx;

    // UNIX socket pipe for passing new sockets from the ListenServer
    int fListenPipe;
    static int sVaultPipe;

public:
    static dsUuid sVaultToken;
    static dsUuid getAuthToken();

public:
    inline dsVaultServer(int pipeFd);
    inline ~dsVaultServer();

    static pid_t Create();
    inline static dsUInt32 RootIdx();
    static void SendConnection(dsSocket* sock);
    static void destroyConnection(dsVaultComm* self);
    static void propagateMessage(const msgparm_t* data, const dsNetMsg* msg);

private:
    void run();
    int getClientConnection();
    dsInt32 InitVault();
};

#include "dsVaultServer.inl"

#endif
