#!/usr/bin/python
import struct, binascii

NodeTypeNames = [ \
    "Invalid (0)", "VNodeMgrLow (1)", "Player (2)", "Age (3)", "GameServer (4)",
    "Admin (5)", "Server (6)", "CCR (7)", "(8)", "(9)", "(10)", "(11)", "(12)",
    "(13)", "(14)", "(15)", "(16)", "(17)", "(18)", "(19)", "(20)",
    "VNodeMgrHigh (21)", "Folder (22)", "PlayerInfo (23)", "System (24)",
    "Image (25)", "TextNote (26)", "SDL (27)", "AgeLink (28)", "Chronicle (29)",
    "PlayerInfoList (30)", "UNUSED (31)", "Marker (32)", "AgeInfo (33)",
    "AgeInfoList (34)", "MarkerList (35)", "(36)", "(37)", "(38)", "(39)",
    ]

def parseUuid(str):
    (d1, d2, d3, d41, d42, d43, d44, d45, d46, d47, d48) = struct.unpack('IHHBBBBBBBB', str)
    return '{0:0=8x}-{1:0=4x}-{2:0=4x}-{3:0=2x}{4:0=2x}-{5:0=2x}{6:0=2x}{7:0=2x}{8:0=2x}{9:0=2x}{10:0=2x}' \
           .format(d1, d2, d3, d41, d42, d43, d44, d45, d46, d47, d48)

def parseNode(str):
    bytes = binascii.a2b_hex(str.replace(' ', ''))
    mask = struct.unpack('Q', bytes[0:8])[0]
    bytes = bytes[8:]
    bit = 1
    while (bit <= mask):
        if ((bit & mask) == 0x00000001):
            print 'Node ID:', struct.unpack('I', bytes[0:4])[0]
            bytes = bytes[4:]
        if ((bit & mask) == 0x00000002):
            print 'Create Time:', struct.unpack('I', bytes[0:4])[0]
            bytes = bytes[4:]
        if ((bit & mask) == 0x00000004):
            print 'Modify Time:', struct.unpack('I', bytes[0:4])[0]
            bytes = bytes[4:]
        if ((bit & mask) == 0x00000008):
            sz = struct.unpack('I', bytes[0:4])[0]
            print 'Create Age Name:', bytes[4:4+sz]
            bytes = bytes[4+sz:]
        if ((bit & mask) == 0x00000010):
            print 'Create Age Uuid:', parseUuid(bytes[0:16])
            bytes = bytes[16:]
        if ((bit & mask) == 0x00000020):
            print 'Creator Uuid:', parseUuid(bytes[0:16])
            bytes = bytes[16:]
        if ((bit & mask) == 0x00000040):
            print 'Creator ID:', struct.unpack('I', bytes[0:4])[0]
            bytes = bytes[4:]
        if ((bit & mask) == 0x00000080):
            print 'Node Type:', NodeTypeNames[struct.unpack('I', bytes[0:4])[0]]
            bytes = bytes[4:]
        if ((bit & mask) == 0x00000100):
            print 'Int32_1:', struct.unpack('i', bytes[0:4])[0]
            bytes = bytes[4:]
        if ((bit & mask) == 0x00000200):
            print 'Int32_2:', struct.unpack('i', bytes[0:4])[0]
            bytes = bytes[4:]
        if ((bit & mask) == 0x00000400):
            print 'Int32_3:', struct.unpack('i', bytes[0:4])[0]
            bytes = bytes[4:]
        if ((bit & mask) == 0x00000800):
            print 'Int32_4:', struct.unpack('i', bytes[0:4])[0]
            bytes = bytes[4:]
        if ((bit & mask) == 0x00001000):
            print 'UInt32_1:', struct.unpack('I', bytes[0:4])[0]
            bytes = bytes[4:]
        if ((bit & mask) == 0x00002000):
            print 'UInt32_2:', struct.unpack('I', bytes[0:4])[0]
            bytes = bytes[4:]
        if ((bit & mask) == 0x00004000):
            print 'UInt32_3:', struct.unpack('I', bytes[0:4])[0]
            bytes = bytes[4:]
        if ((bit & mask) == 0x00008000):
            print 'UInt32_4:', struct.unpack('I', bytes[0:4])[0]
            bytes = bytes[4:]
        if ((bit & mask) == 0x00010000):
            print 'Uuid_1:', parseUuid(bytes[0:16])
            bytes = bytes[16:]
        if ((bit & mask) == 0x00020000):
            print 'Uuid_2:', parseUuid(bytes[0:16])
            bytes = bytes[16:]
        if ((bit & mask) == 0x00040000):
            print 'Uuid_3:', parseUuid(bytes[0:16])
            bytes = bytes[16:]
        if ((bit & mask) == 0x00080000):
            print 'Uuid_4:', parseUuid(bytes[0:16])
            bytes = bytes[16:]
        if ((bit & mask) == 0x00100000):
            sz = struct.unpack('I', bytes[0:4])[0]
            print 'String64_1:', bytes[4:4+sz]
            bytes = bytes[4+sz:]
        if ((bit & mask) == 0x00200000):
            sz = struct.unpack('I', bytes[0:4])[0]
            print 'String64_2:', bytes[4:4+sz]
            bytes = bytes[4+sz:]
        if ((bit & mask) == 0x00400000):
            sz = struct.unpack('I', bytes[0:4])[0]
            print 'String64_3:', bytes[4:4+sz]
            bytes = bytes[4+sz:]
        if ((bit & mask) == 0x00800000):
            sz = struct.unpack('I', bytes[0:4])[0]
            print 'String64_4:', bytes[4:4+sz]
            bytes = bytes[4+sz:]
        if ((bit & mask) == 0x01000000):
            sz = struct.unpack('I', bytes[0:4])[0]
            print 'String64_5:', bytes[4:4+sz]
            bytes = bytes[4+sz:]
        if ((bit & mask) == 0x02000000):
            sz = struct.unpack('I', bytes[0:4])[0]
            print 'String64_6:', bytes[4:4+sz]
            bytes = bytes[4+sz:]
        if ((bit & mask) == 0x04000000):
            sz = struct.unpack('I', bytes[0:4])[0]
            print 'IString64_1:', bytes[4:4+sz]
            bytes = bytes[4+sz:]
        if ((bit & mask) == 0x08000000):
            sz = struct.unpack('I', bytes[0:4])[0]
            print 'IString64_2:', bytes[4:4+sz]
            bytes = bytes[4+sz:]
        if ((bit & mask) == 0x10000000):
            sz = struct.unpack('I', bytes[0:4])[0]
            print 'Text_1:', bytes[4:4+sz]
            bytes = bytes[4+sz:]
        if ((bit & mask) == 0x20000000):
            sz = struct.unpack('I', bytes[0:4])[0]
            print 'Text_2:', bytes[4:4+sz]
            bytes = bytes[4+sz:]
        if ((bit & mask) == 0x40000000):
            sz = struct.unpack('I', bytes[0:4])[0]
            print 'Blob_1:', binascii.b2a_hex(bytes[4:4+sz])
            bytes = bytes[4+sz:]
        if ((bit & mask) == 0x80000000):
            sz = struct.unpack('I', bytes[0:4])[0]
            print 'Blob_2:', binascii.b2a_hex(bytes[4:4+sz])
            bytes = bytes[4+sz:]
        bit <<= 1
    if (bytes != ''):
        print "LEFTOVERS:", repr(bytes)

if __name__ == '__main__':
    import string, sys
    parseNode(string.join(sys.argv[1:]))
